@@ -47,8 +47,12 @@ impl Internal {
             Internal::Dong => include_bytes!("../../sounds/dong.ogg").to_vec(),
             Internal::Peck => include_bytes!("../../sounds/peck.ogg").to_vec(),
             Internal::Ring => include_bytes!("../../sounds/ring.ogg").to_vec(),
-            Internal::Squeak => include_bytes!("../../sounds/squeak.ogg").to_vec(),
-            Internal::Whistle => include_bytes!("../../sounds/whistle.ogg").to_vec(),
+            Internal::Squeak => {
+                include_bytes!("../../sounds/squeak.ogg").to_vec()
+            }
+            Internal::Whistle => {
+                include_bytes!("../../sounds/whistle.ogg").to_vec()
+            }
             Internal::Bonk => include_bytes!("../../sounds/bonk.ogg").to_vec(),
             Internal::Sing => include_bytes!("../../sounds/sing.ogg").to_vec(),
         }
@@ -77,18 +81,21 @@ fn find_external_sound(sound: &str) -> Result<PathBuf, LoadError> {
 
     for e in walkdir::WalkDir::new(sounds_dir.clone())
         .into_iter()
-        .filter_map(|e| e.ok())
+        .filter_map(Result::ok)
     {
-        if e.metadata().map(|data| data.is_file()).unwrap_or_default() && e.file_name() == sound {
+        if e.metadata().is_ok_and(|data| data.is_file())
+            && e.file_name() == sound
+        {
             return Ok(e.path().to_path_buf());
         }
     }
 
-    let sounds_dir = if let Ok(sounds_dir) = sounds_dir.into_os_string().into_string() {
-        format!(" in {sounds_dir}")
-    } else {
-        String::new()
-    };
+    let sounds_dir =
+        if let Ok(sounds_dir) = sounds_dir.into_os_string().into_string() {
+            format!(" in {sounds_dir}")
+        } else {
+            String::new()
+        };
 
     Err(LoadError::NoSoundFound(sound.to_string(), sounds_dir))
 }
