@@ -2,9 +2,9 @@ use core::fmt;
 
 use serde::{Deserialize, Serialize};
 
-use crate::config::buffer::NicknameClickAction;
+use crate::serde::deserialize_strftime_date;
 use crate::target::{self, Target};
-use crate::{channel, config, message, Server};
+use crate::{Server, channel, config, message};
 
 #[derive(Debug, Clone, PartialEq, Eq, Hash, Serialize, Deserialize)]
 #[serde(untagged)]
@@ -20,7 +20,17 @@ pub enum Upstream {
     Query(Server, target::Query),
 }
 
-#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash, Serialize, Deserialize, strum::Display)]
+#[derive(
+    Debug,
+    Clone,
+    Copy,
+    PartialEq,
+    Eq,
+    Hash,
+    Serialize,
+    Deserialize,
+    strum::Display,
+)]
 pub enum Internal {
     #[strum(serialize = "File Transfers")]
     FileTransfers,
@@ -68,7 +78,9 @@ impl Upstream {
 
     pub fn server(&self) -> &Server {
         match self {
-            Self::Server(server) | Self::Channel(server, _) | Self::Query(server, _) => server,
+            Self::Server(server)
+            | Self::Channel(server, _)
+            | Self::Query(server, _) => server,
         }
     }
 
@@ -87,7 +99,10 @@ impl Upstream {
         }
     }
 
-    pub fn server_message_target(self, source: Option<message::source::Server>) -> message::Target {
+    pub fn server_message_target(
+        self,
+        source: Option<message::source::Server>,
+    ) -> message::Target {
         match self {
             Self::Server(_) => message::Target::Server {
                 source: message::Source::Server(source),
@@ -105,7 +120,8 @@ impl Upstream {
 }
 
 impl Internal {
-    pub const ALL: &'static [Self] = &[Self::FileTransfers, Self::Logs, Self::Highlights];
+    pub const ALL: &'static [Self] =
+        &[Self::FileTransfers, Self::Logs, Self::Highlights];
 
     pub fn key(&self) -> String {
         match self {
@@ -131,15 +147,21 @@ impl From<config::Buffer> for Settings {
 }
 
 #[derive(Debug, Clone, Default, Deserialize)]
+#[serde(default)]
 pub struct TextInput {
-    #[serde(default)]
     pub visibility: TextInputVisibility,
-    #[serde(default)]
     pub auto_format: AutoFormat,
-    #[serde(default)]
     pub autocomplete: Autocomplete,
 }
 
+#[derive(Debug, Clone, Copy, Default, Deserialize)]
+#[serde(rename_all = "kebab-case")]
+pub enum OrderBy {
+    Alpha,
+    #[default]
+    Recent,
+}
+
 #[derive(Debug, Clone, Copy, Default, Deserialize)]
 #[serde(rename_all = "kebab-case")]
 pub enum SortDirection {
@@ -149,18 +171,19 @@ pub enum SortDirection {
 }
 
 #[derive(Debug, Clone, Deserialize)]
+#[serde(default)]
 pub struct Autocomplete {
-    #[serde(default)]
+    pub order_by: OrderBy,
     pub sort_direction: SortDirection,
-    #[serde(default = "default_completion_suffixes")]
     pub completion_suffixes: [String; 2],
 }
 
 impl Default for Autocomplete {
     fn default() -> Self {
         Self {
+            order_by: OrderBy::default(),
             sort_direction: SortDirection::default(),
-            completion_suffixes: default_completion_suffixes(),
+            completion_suffixes: [": ".to_string(), " ".to_string()],
         }
     }
 }
@@ -183,68 +206,48 @@ pub enum AutoFormat {
 }
 
 #[derive(Debug, Clone, Deserialize)]
+#[serde(default)]
 pub struct Timestamp {
-    #[serde(default = "default_timestamp")]
+    #[serde(deserialize_with = "deserialize_strftime_date")]
     pub format: String,
-    #[serde(default)]
     pub brackets: Brackets,
 }
 
 impl Default for Timestamp {
     fn default() -> Self {
         Self {
-            format: default_timestamp(),
-            brackets: Default::default(),
+            format: "%R".to_string(),
+            brackets: Brackets::default(),
         }
     }
 }
 
+#[derive(Debug, Clone, Default, Deserialize)]
+#[serde(default)]
+pub struct BacklogSeparator {
+    pub hide_when_all_read: bool,
+}
+
 #[derive(Debug, Clone, Deserialize)]
+#[serde(default)]
 pub struct DateSeparators {
-    #[serde(default = "default_date_separators")]
+    #[serde(deserialize_with = "deserialize_strftime_date")]
     pub format: String,
-    #[serde(default = "default_bool_true")]
     pub show: bool,
 }
 
 impl Default for DateSeparators {
     fn default() -> Self {
         Self {
-            format: default_date_separators(),
+            format: "%A, %B %-d".to_string(),
             show: true,
         }
     }
 }
 
-#[derive(Debug, Clone, Deserialize)]
-pub struct Nickname {
-    #[serde(default)]
-    pub color: Color,
-    #[serde(default)]
-    pub brackets: Brackets,
-    #[serde(default)]
-    pub alignment: Alignment,
-    #[serde(default = "default_bool_true")]
-    pub show_access_levels: bool,
-    #[serde(default)]
-    pub click: NicknameClickAction,
-}
-
-impl Default for Nickname {
-    fn default() -> Self {
-        Self {
-            color: Default::default(),
-            brackets: Default::default(),
-            alignment: Default::default(),
-            show_access_levels: default_bool_true(),
-            click: Default::default(),
-        }
-    }
-}
-
 #[derive(Debug, Clone, Default, Deserialize)]
+#[serde(default)]
 pub struct StatusMessagePrefix {
-    #[serde(default)]
     pub brackets: Brackets,
 }
 
@@ -328,19 +331,3 @@ impl From<SkinTone> for emojis::SkinTone {
         }
     }
 }
-
-fn default_timestamp() -> String {
-    "%R".to_string()
-}
-
-fn default_date_separators() -> String {
-    "%A, %B %-d".to_string()
-}
-
-fn default_bool_true() -> bool {
-    true
-}
-
-fn default_completion_suffixes() -> [String; 2] {
-    [": ".to_string(), " ".to_string()]
-}
