@@ -1,43 +1,36 @@
 use chrono::{DateTime, Local, Utc};
-use serde::Deserialize;
+use serde::{Deserialize, Deserializer};
 
-pub use self::away::Away;
 pub use self::channel::Channel;
+use crate::config::buffer::nickname::Nickname;
 
-pub mod away;
 pub mod channel;
+pub mod nickname;
 
-use crate::{
-    buffer::{DateSeparators, Nickname, SkinTone, StatusMessagePrefix, TextInput, Timestamp},
-    message::source,
+use crate::buffer::{
+    BacklogSeparator, DateSeparators, SkinTone, StatusMessagePrefix, TextInput,
+    Timestamp,
 };
+use crate::message::source;
 
-#[derive(Debug, Clone, Deserialize, Default)]
+#[derive(Debug, Default, Clone, Deserialize)]
+#[serde(default)]
 pub struct Buffer {
-    #[serde(default)]
-    pub away: Away,
-    #[serde(default)]
     pub timestamp: Timestamp,
-    #[serde(default)]
     pub nickname: Nickname,
-    #[serde(default)]
     pub text_input: TextInput,
-    #[serde(default)]
     pub channel: Channel,
-    #[serde(default)]
     pub server_messages: ServerMessages,
-    #[serde(default)]
     pub internal_messages: InternalMessages,
-    #[serde(default)]
     pub status_message_prefix: StatusMessagePrefix,
-    #[serde(default)]
     pub chathistory: ChatHistory,
-    #[serde(default)]
+    pub backlog_separator: BacklogSeparator,
     pub date_separators: DateSeparators,
-    #[serde(default)]
     pub commands: Commands,
-    #[serde(default)]
     pub emojis: Emojis,
+    pub mark_as_read: MarkAsRead,
+    pub url: Url,
+    pub line_spacing: u32,
 }
 
 #[derive(Debug, Clone, Deserialize, Default)]
@@ -49,61 +42,162 @@ pub enum NicknameClickAction {
 }
 
 #[derive(Debug, Clone, Deserialize)]
+#[serde(default)]
 pub struct Emojis {
-    #[serde(default = "default_bool_true")]
     pub show_picker: bool,
-    #[serde(default)]
     pub skin_tone: SkinTone,
-    #[serde(default = "default_bool_true")]
     pub auto_replace: bool,
+    pub characters_to_trigger_picker: usize,
 }
 
 impl Default for Emojis {
     fn default() -> Self {
         Self {
-            show_picker: default_bool_true(),
-            skin_tone: Default::default(),
-            auto_replace: default_bool_true(),
+            show_picker: true,
+            skin_tone: SkinTone::default(),
+            auto_replace: true,
+            characters_to_trigger_picker: 2,
+        }
+    }
+}
+
+#[derive(Debug, Clone, Deserialize, Default)]
+#[serde(default)]
+pub struct Url {
+    pub prompt_before_open: bool,
+}
+
+#[derive(Debug, Clone, Deserialize)]
+#[serde(default)]
+pub struct MarkAsRead {
+    pub on_application_exit: bool,
+    pub on_buffer_close: OnBufferClose,
+    pub on_scroll_to_bottom: bool,
+    pub on_message_sent: bool,
+}
+
+impl Default for MarkAsRead {
+    fn default() -> Self {
+        Self {
+            on_application_exit: false,
+            on_buffer_close: OnBufferClose::default(),
+            on_scroll_to_bottom: true,
+            on_message_sent: true,
         }
     }
 }
 
 #[derive(Debug, Clone, Deserialize)]
+#[serde(untagged)]
+pub enum OnBufferClose {
+    Bool(bool),
+    Condition(OnBufferCloseCondition),
+}
+
+impl Default for OnBufferClose {
+    fn default() -> Self {
+        Self::Condition(OnBufferCloseCondition::ScrolledToBottom)
+    }
+}
+
+impl OnBufferClose {
+    pub fn mark_as_read(&self, is_scrolled_to_bottom: Option<bool>) -> bool {
+        match self {
+            OnBufferClose::Bool(mark) => *mark,
+            OnBufferClose::Condition(_) => {
+                is_scrolled_to_bottom.unwrap_or(false)
+            }
+        }
+    }
+}
+
+#[derive(Debug, Clone, Deserialize)]
+#[serde(rename_all = "kebab-case")]
+pub enum OnBufferCloseCondition {
+    ScrolledToBottom,
+}
+
+#[derive(Debug, Clone, Deserialize)]
+#[serde(default)]
 pub struct Commands {
-    #[serde(default = "default_bool_true")]
     pub show_description: bool,
 }
 
 impl Default for Commands {
     fn default() -> Self {
         Self {
-            show_description: default_bool_true(),
+            show_description: true,
+        }
+    }
+}
+
+#[derive(Debug, Clone, Copy)]
+pub enum Away {
+    Dimmed(Option<f32>),
+    None,
+}
+
+impl Away {
+    pub fn is_away(&self, is_user_away: bool) -> Option<Away> {
+        is_user_away.then_some(*self)
+    }
+}
+
+impl Default for Away {
+    fn default() -> Self {
+        Away::Dimmed(None)
+    }
+}
+
+impl<'de> Deserialize<'de> for Away {
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+    where
+        D: Deserializer<'de>,
+    {
+        #[derive(Deserialize)]
+        #[serde(untagged)]
+        enum AppearanceRepr {
+            String(String),
+            Struct(DimmedStruct),
+        }
+
+        #[derive(Deserialize)]
+        struct DimmedStruct {
+            dimmed: Option<f32>,
+        }
+
+        let repr = AppearanceRepr::deserialize(deserializer)?;
+        match repr {
+            AppearanceRepr::String(s) => match s.as_str() {
+                "dimmed" => Ok(Away::Dimmed(None)),
+                "solid" | "none" => Ok(Away::None),
+                _ => Err(serde::de::Error::custom(format!(
+                    "unknown appearance: {s}",
+                ))),
+            },
+            AppearanceRepr::Struct(s) => Ok(Away::Dimmed(s.dimmed)),
         }
     }
 }
 
 #[derive(Debug, Clone, Default, Deserialize)]
+#[serde(default)]
 pub struct ServerMessages {
-    #[serde(default)]
     pub topic: ServerMessage,
-    #[serde(default)]
     pub join: ServerMessage,
-    #[serde(default)]
     pub part: ServerMessage,
-    #[serde(default)]
     pub quit: ServerMessage,
-    #[serde(default)]
     pub change_host: ServerMessage,
-    #[serde(default)]
+    pub change_mode: ServerMessage,
+    pub change_nick: ServerMessage,
     pub monitored_online: ServerMessage,
-    #[serde(default)]
     pub monitored_offline: ServerMessage,
-    #[serde(default)]
     pub standard_reply_fail: ServerMessage,
-    #[serde(default)]
     pub standard_reply_warn: ServerMessage,
-    #[serde(default)]
     pub standard_reply_note: ServerMessage,
+    pub wallops: ServerMessage,
+    pub kick: ServerMessage,
+    pub change_topic: ServerMessage,
 }
 
 impl ServerMessages {
@@ -114,32 +208,37 @@ impl ServerMessages {
             source::server::Kind::Part => Some(&self.part),
             source::server::Kind::Quit => Some(&self.quit),
             source::server::Kind::ChangeHost => Some(&self.change_host),
-            source::server::Kind::MonitoredOnline => Some(&self.monitored_online),
-            source::server::Kind::MonitoredOffline => Some(&self.monitored_offline),
-            source::server::Kind::StandardReply(source::server::StandardReply::Fail) => {
-                Some(&self.standard_reply_fail)
+            source::server::Kind::ChangeMode => Some(&self.change_mode),
+            source::server::Kind::ChangeNick => Some(&self.change_nick),
+            source::server::Kind::MonitoredOnline => {
+                Some(&self.monitored_online)
             }
-            source::server::Kind::StandardReply(source::server::StandardReply::Warn) => {
-                Some(&self.standard_reply_warn)
-            }
-            source::server::Kind::StandardReply(source::server::StandardReply::Note) => {
-                Some(&self.standard_reply_note)
+            source::server::Kind::MonitoredOffline => {
+                Some(&self.monitored_offline)
             }
+            source::server::Kind::StandardReply(
+                source::server::StandardReply::Fail,
+            ) => Some(&self.standard_reply_fail),
+            source::server::Kind::StandardReply(
+                source::server::StandardReply::Warn,
+            ) => Some(&self.standard_reply_warn),
+            source::server::Kind::StandardReply(
+                source::server::StandardReply::Note,
+            ) => Some(&self.standard_reply_note),
+            source::server::Kind::WAllOps => Some(&self.wallops),
+            source::server::Kind::Kick => Some(&self.kick),
+            source::server::Kind::ChangeTopic => Some(&self.change_topic),
         }
     }
 }
 
 #[derive(Debug, Clone, Deserialize)]
+#[serde(default)]
 pub struct ServerMessage {
-    #[serde(default = "default_bool_true")]
     pub enabled: bool,
-    #[serde(default)]
     pub smart: Option<i64>,
-    #[serde(default)]
     pub username_format: UsernameFormat,
-    #[serde(default)]
     pub exclude: Vec<String>,
-    #[serde(default)]
     pub include: Vec<String>,
 }
 
@@ -147,10 +246,10 @@ impl Default for ServerMessage {
     fn default() -> Self {
         Self {
             enabled: true,
-            smart: Default::default(),
+            smart: None,
             username_format: UsernameFormat::default(),
-            exclude: Default::default(),
-            include: Default::default(),
+            exclude: Vec::default(),
+            include: Vec::default(),
         }
     }
 }
@@ -165,8 +264,9 @@ impl ServerMessage {
         let is_channel_filtered = |list: &Vec<String>, channel: &str| -> bool {
             let wildcards = ["*", "all"];
 
-            list.iter()
-                .any(|item| wildcards.contains(&item.as_str()) || item == channel)
+            list.iter().any(|item| {
+                wildcards.contains(&item.as_str()) || item == channel
+            })
         };
 
         let channel_included = is_channel_filtered(&self.include, channel);
@@ -178,10 +278,9 @@ impl ServerMessage {
 }
 
 #[derive(Debug, Clone, Default, Deserialize)]
+#[serde(default)]
 pub struct InternalMessages {
-    #[serde(default)]
     pub success: InternalMessage,
-    #[serde(default)]
     pub error: InternalMessage,
 }
 
@@ -195,10 +294,9 @@ impl InternalMessages {
 }
 
 #[derive(Debug, Clone, Deserialize)]
+#[serde(default)]
 pub struct InternalMessage {
-    #[serde(default = "default_bool_true")]
     pub enabled: bool,
-    #[serde(default)]
     pub smart: Option<i64>,
 }
 
@@ -206,14 +304,26 @@ impl Default for InternalMessage {
     fn default() -> Self {
         Self {
             enabled: true,
-            smart: Default::default(),
+            smart: None,
         }
     }
 }
 
+#[derive(Debug, Default, Clone, Deserialize)]
+#[serde(rename_all = "kebab-case")]
+pub enum LevelFilter {
+    Off,
+    Error,
+    Warn,
+    #[default]
+    Info,
+    Debug,
+    Trace,
+}
+
 #[derive(Debug, Clone, Deserialize)]
+#[serde(default)]
 pub struct ChatHistory {
-    #[serde(default = "default_bool_true")]
     pub infinite_scroll: bool,
 }
 
@@ -231,10 +341,15 @@ pub enum UsernameFormat {
     Short,
     #[default]
     Full,
+    #[serde(skip)]
+    Mask,
 }
 
 impl Buffer {
-    pub fn format_timestamp(&self, date_time: &DateTime<Utc>) -> Option<String> {
+    pub fn format_timestamp(
+        &self,
+        date_time: &DateTime<Utc>,
+    ) -> Option<String> {
         if self.timestamp.format.is_empty() {
             return None;
         }
@@ -249,7 +364,3 @@ impl Buffer {
         ))
     }
 }
-
-fn default_bool_true() -> bool {
-    true
-}
