@@ -1,18 +1,26 @@
-use std::{net::IpAddr, num::NonZeroU16, ops::RangeInclusive, path::PathBuf};
+use std::net::IpAddr;
+use std::num::NonZeroU16;
+use std::ops::RangeInclusive;
+use std::path::PathBuf;
 
 use serde::Deserialize;
 
+use crate::serde::deserialize_path_buf_with_path_transformations_maybe;
+
 #[derive(Debug, Clone, Deserialize)]
+#[serde(default)]
 pub struct FileTransfer {
     /// Default directory to save files in. If not set, user will see a file dialog.
-    #[serde(default)]
+    #[serde(
+        deserialize_with = "deserialize_path_buf_with_path_transformations_maybe"
+    )]
     pub save_directory: Option<PathBuf>,
     /// If true, act as the "client" for the transfer. Requires the remote user act as the server.
-    #[serde(default = "default_passive")]
     pub passive: bool,
     /// Time in seconds to wait before timing out a transfer waiting to be accepted.
-    #[serde(default = "default_timeout")]
     pub timeout: u64,
+    /// Auto-accept configuration for incoming file transfers.
+    pub auto_accept: AutoAccept,
     pub server: Option<Server>,
 }
 
@@ -20,19 +28,23 @@ impl Default for FileTransfer {
     fn default() -> Self {
         Self {
             save_directory: None,
-            passive: default_passive(),
-            timeout: default_timeout(),
+            passive: true,
+            timeout: 60 * 5,
+            auto_accept: AutoAccept::default(),
             server: None,
         }
     }
 }
 
-fn default_passive() -> bool {
-    true
-}
-
-fn default_timeout() -> u64 {
-    60 * 5
+#[derive(Debug, Clone, Default, Deserialize)]
+#[serde(default)]
+pub struct AutoAccept {
+    /// If true, automatically accept incoming file transfers. Requires save_directory to be set.
+    pub enabled: bool,
+    /// Auto-accept incoming file transfers from these nicks. Requires enabled to be true.
+    pub nicks: Option<Vec<String>>,
+    /// Auto-accept incoming file transfers from these masks (regex patterns). Requires enabled to be true.
+    pub masks: Option<Vec<String>>,
 }
 
 #[derive(Debug, Clone)]
