@@ -6,21 +6,33 @@ use irc::connection;
 use serde::{Deserialize, Deserializer};
 
 use crate::config;
+use crate::serde::{
+    deserialize_path_buf_with_path_transformations,
+    deserialize_path_buf_with_path_transformations_maybe,
+};
+
+const DEFAULT_PORT: u16 = 6667;
+const DEFAULT_TLS_PORT: u16 = 6697;
 
 #[derive(Debug, Clone, PartialEq, Eq, Deserialize)]
+#[serde(default)]
 pub struct Server {
     /// The client's nickname.
     pub nickname: String,
     /// The client's NICKSERV password.
     pub nick_password: Option<String>,
     /// The client's NICKSERV password file.
-    pub nick_password_file: Option<String>,
+    #[serde(
+        deserialize_with = "deserialize_path_buf_with_path_transformations_maybe"
+    )]
+    pub nick_password_file: Option<PathBuf>,
+    /// Truncate read from NICKSERV password file to first newline
+    pub nick_password_file_first_line_only: bool,
     /// The client's NICKSERV password command.
     pub nick_password_command: Option<String>,
     /// The server's NICKSERV IDENTIFY syntax.
     pub nick_identify_syntax: Option<IdentifySyntax>,
     /// Alternative nicknames for the client, if the default is taken.
-    #[serde(default)]
     pub alt_nicks: Vec<String>,
     /// The client's username.
     pub username: Option<String>,
@@ -29,69 +41,64 @@ pub struct Server {
     /// The server to connect to.
     pub server: String,
     /// The port to connect on.
-    #[serde(default = "default_tls_port")]
     pub port: u16,
     /// The password to connect to the server.
     pub password: Option<String>,
     /// The file with the password to connect to the server.
-    pub password_file: Option<String>,
+    #[serde(
+        deserialize_with = "deserialize_path_buf_with_path_transformations_maybe"
+    )]
+    pub password_file: Option<PathBuf>,
+    /// Truncate read from password file to first newline
+    pub password_file_first_line_only: bool,
     /// The command which outputs a password to connect to the server.
     pub password_command: Option<String>,
+    /// Filter settings for the server, e.g. ignored nicks
+    pub filters: Option<Filters>,
     /// A list of channels to join on connection.
-    #[serde(default)]
     pub channels: Vec<String>,
     /// A mapping of channel names to keys for join-on-connect.
-    #[serde(default)]
     pub channel_keys: HashMap<String, String>,
     /// The amount of inactivity in seconds before the client will ping the server.
-    #[serde(default = "default_ping_time")]
     pub ping_time: u64,
     /// The amount of time in seconds for a client to reconnect due to no ping response.
-    #[serde(default = "default_ping_timeout")]
     pub ping_timeout: u64,
     /// The amount of time in seconds before attempting to reconnect to the server when disconnected.
-    #[serde(default = "default_reconnect_delay")]
     pub reconnect_delay: u64,
     /// Whether the client should use NickServ GHOST to reclaim its primary nickname if it is in
     /// use. This has no effect if `nick_password` is not set.
-    #[serde(default)]
     pub should_ghost: bool,
     /// The command(s) that should be sent to NickServ to recover a nickname. The nickname and
     /// password will be appended in that order after the command.
     /// E.g. `["RECOVER", "RELEASE"]` means `RECOVER nick pass` and `RELEASE nick pass` will be sent
     /// in that order.
-    #[serde(default = "default_ghost_sequence")]
     pub ghost_sequence: Vec<String>,
     /// User modestring to set on connect. Example: "+RB-x"
     pub umodes: Option<String>,
     /// Whether or not to use TLS.
     /// Clients will automatically panic if this is enabled without TLS support.
-    #[serde(default = "default_use_tls")]
     pub use_tls: bool,
     /// On `true`, all certificate validations are skipped. Defaults to `false`.
-    #[serde(default)]
     pub dangerously_accept_invalid_certs: bool,
     /// The path to the root TLS certificate for this server in PEM format.
+    #[serde(
+        deserialize_with = "deserialize_path_buf_with_path_transformations_maybe"
+    )]
     root_cert_path: Option<PathBuf>,
     /// Sasl authentication
     pub sasl: Option<Sasl>,
     /// Commands which are executed once connected.
-    #[serde(default)]
     pub on_connect: Vec<String>,
     /// Enable WHO polling. Defaults to `true`.
-    #[serde(default = "default_who_poll_enabled")]
     pub who_poll_enabled: bool,
     /// WHO poll interval for servers without away-notify.
-    #[serde(
-        default = "default_who_poll_interval",
-        deserialize_with = "deserialize_duration_from_u64"
-    )]
+    #[serde(deserialize_with = "deserialize_duration_from_u64")]
     pub who_poll_interval: Duration,
     /// A list of nicknames to monitor (if MONITOR is supported by the server).
-    #[serde(default)]
     pub monitor: Vec<String>,
-    #[serde(default = "default_chathistory")]
     pub chathistory: bool,
+    #[serde(skip)]
+    pub order: u16,
 }
 
 impl Server {
@@ -106,9 +113,9 @@ impl Server {
             nickname,
             server,
             port: port.unwrap_or(if use_tls {
-                default_tls_port()
+                DEFAULT_TLS_PORT
             } else {
-                default_port()
+                DEFAULT_PORT
             }),
             channels,
             use_tls,
@@ -117,13 +124,22 @@ impl Server {
         }
     }
 
-    pub fn connection(&self, proxy: Option<config::Proxy>) -> connection::Config {
+    pub fn connection(
+        &self,
+        proxy: Option<config::Proxy>,
+    ) -> connection::Config<'_> {
         let security = if self.use_tls {
             connection::Security::Secured {
                 accept_invalid_certs: self.dangerously_accept_invalid_certs,
                 root_cert_path: self.root_cert_path.as_ref(),
-                client_cert_path: self.sasl.as_ref().and_then(Sasl::external_cert),
-                client_key_path: self.sasl.as_ref().and_then(Sasl::external_key),
+                client_cert_path: self
+                    .sasl
+                    .as_ref()
+                    .and_then(Sasl::external_cert),
+                client_key_path: self
+                    .sasl
+                    .as_ref()
+                    .and_then(Sasl::external_key),
             }
         } else {
             connection::Security::Unsecured
@@ -136,41 +152,64 @@ impl Server {
             proxy: proxy.map(From::from),
         }
     }
+
+    pub fn bouncer_config(&self) -> Self {
+        Self {
+            // nickserv info not relevant to the bounced network
+            nick_password_file: Option::default(),
+            nick_password_command: Option::default(),
+            nick_identify_syntax: Option::default(),
+
+            // channels not relevant
+            channels: Vec::default(),
+            channel_keys: HashMap::default(),
+
+            // ghost sequence not relevant
+            should_ghost: Default::default(),
+            ghost_sequence: Server::default().ghost_sequence,
+
+            ..self.clone()
+        }
+    }
 }
 
 impl Default for Server {
     fn default() -> Self {
         Self {
-            nickname: Default::default(),
-            nick_password: Default::default(),
-            nick_password_file: Default::default(),
-            nick_password_command: Default::default(),
-            nick_identify_syntax: Default::default(),
-            alt_nicks: Default::default(),
-            username: Default::default(),
-            realname: Default::default(),
-            server: Default::default(),
-            port: default_tls_port(),
-            password: Default::default(),
-            password_file: Default::default(),
-            password_command: Default::default(),
-            channels: Default::default(),
-            channel_keys: Default::default(),
-            ping_time: default_ping_time(),
-            ping_timeout: default_ping_timeout(),
-            reconnect_delay: default_reconnect_delay(),
+            nickname: String::default(),
+            nick_password: Option::default(),
+            nick_password_file: Option::default(),
+            nick_password_file_first_line_only: true,
+            nick_password_command: Option::default(),
+            nick_identify_syntax: Option::default(),
+            alt_nicks: Vec::default(),
+            username: Option::default(),
+            realname: Option::default(),
+            server: String::default(),
+            port: DEFAULT_TLS_PORT,
+            password: Option::default(),
+            password_file: Option::default(),
+            password_file_first_line_only: true,
+            password_command: Option::default(),
+            filters: Option::default(),
+            channels: Vec::default(),
+            channel_keys: HashMap::default(),
+            ping_time: 180,
+            ping_timeout: 20,
+            reconnect_delay: 10,
             should_ghost: Default::default(),
-            ghost_sequence: default_ghost_sequence(),
-            umodes: Default::default(),
-            use_tls: default_use_tls(),
+            ghost_sequence: vec!["REGAIN".into()],
+            umodes: Option::default(),
+            use_tls: true,
             dangerously_accept_invalid_certs: Default::default(),
-            root_cert_path: Default::default(),
-            sasl: Default::default(),
-            on_connect: Default::default(),
-            who_poll_enabled: default_who_poll_enabled(),
-            who_poll_interval: default_who_poll_interval(),
-            monitor: Default::default(),
-            chathistory: default_chathistory(),
+            root_cert_path: Option::default(),
+            sasl: Option::default(),
+            on_connect: Vec::default(),
+            who_poll_enabled: true,
+            who_poll_interval: Duration::from_secs(2),
+            monitor: Vec::default(),
+            chathistory: true,
+            order: 0,
         }
     }
 }
@@ -191,14 +230,27 @@ pub enum Sasl {
         /// Account password,
         password: Option<String>,
         /// Account password file
-        password_file: Option<String>,
+        #[serde(
+            default,
+            deserialize_with = "deserialize_path_buf_with_path_transformations_maybe"
+        )]
+        password_file: Option<PathBuf>,
+        /// Truncate read from password file to first newline
+        password_file_first_line_only: Option<bool>,
         /// Account password command
         password_command: Option<String>,
     },
     External {
         /// The path to PEM encoded X509 user certificate for external auth
+        #[serde(
+            deserialize_with = "deserialize_path_buf_with_path_transformations"
+        )]
         cert: PathBuf,
         /// The path to PEM encoded PKCS#8 private key corresponding to the user certificate for external auth
+        #[serde(
+            default,
+            deserialize_with = "deserialize_path_buf_with_path_transformations_maybe"
+        )]
         key: Option<PathBuf>,
     },
 }
@@ -242,12 +294,13 @@ impl Sasl {
                 let mut params = chunks
                     .into_iter()
                     .map(|chunk| {
-                        String::from_utf8(chunk.into()).expect("chunks should be valid UTF-8")
+                        String::from_utf8(chunk.into())
+                            .expect("chunks should be valid UTF-8")
                     })
                     .collect::<Vec<String>>();
 
                 if signal_end_of_response {
-                    params.push("+".into())
+                    params.push("+".into());
                 }
 
                 params
@@ -273,50 +326,18 @@ impl Sasl {
     }
 }
 
-fn deserialize_duration_from_u64<'de, D>(deserializer: D) -> Result<Duration, D::Error>
+#[derive(PartialEq, Eq, Debug, Clone, Deserialize, Default)]
+#[serde(default)]
+pub struct Filters {
+    pub ignore: Vec<String>,
+}
+
+fn deserialize_duration_from_u64<'de, D>(
+    deserializer: D,
+) -> Result<Duration, D::Error>
 where
     D: Deserializer<'de>,
 {
     let seconds: u64 = Deserialize::deserialize(deserializer)?;
     Ok(Duration::from_secs(seconds.clamp(1, 3600)))
 }
-
-fn default_use_tls() -> bool {
-    true
-}
-
-fn default_tls_port() -> u16 {
-    6697
-}
-
-fn default_port() -> u16 {
-    6667
-}
-
-fn default_ping_time() -> u64 {
-    180
-}
-
-fn default_ping_timeout() -> u64 {
-    20
-}
-
-fn default_reconnect_delay() -> u64 {
-    10
-}
-
-fn default_ghost_sequence() -> Vec<String> {
-    vec!["REGAIN".into()]
-}
-
-fn default_who_poll_enabled() -> bool {
-    true
-}
-
-fn default_who_poll_interval() -> Duration {
-    Duration::from_secs(2)
-}
-
-fn default_chathistory() -> bool {
-    true
-}
