@@ -1,12 +1,11 @@
-use std::{
-    net::{IpAddr, Ipv4Addr},
-    num::NonZeroU16,
-};
+use std::net::{IpAddr, Ipv4Addr};
+use std::num::NonZeroU16;
 
-use crate::ctcp;
 use irc::proto;
 use itertools::Itertools;
 
+use crate::ctcp;
+
 pub fn decode(content: &str) -> Option<Command> {
     let query = ctcp::parse_query(content)?;
 
@@ -75,7 +74,7 @@ impl Send {
         }
 
         // Host will always be 3rd or 4th arg in reverse order
-        // The last arg to succesfully decode as host will be host
+        // The last arg to successfully decode as host will be host
         let host_pos = args.len()
             - 1
             - args
@@ -128,12 +127,14 @@ impl Send {
                 token,
             } => {
                 let host = encode_host(host);
-                let port = port.map(NonZeroU16::get).unwrap_or(0);
+                let port = port.map_or(0, NonZeroU16::get);
 
                 ctcp::query_message(
                     &ctcp::Command::DCC,
                     target.to_string(),
-                    Some(format!("SEND {filename} {host} {port} {size} {token}")),
+                    Some(format!(
+                        "SEND {filename} {host} {port} {size} {token}"
+                    )),
                 )
             }
             Self::Direct {
