@@ -7,8 +7,10 @@ pub const CONFIG_FILE_NAME: &str = "config.toml";
 pub const APPLICATION_ID: &str = "org.squidowl.halloy";
 pub const WIKI_WEBSITE: &str = "https://halloy.chat";
 pub const THEME_WEBSITE: &str = "https://themes.halloy.chat";
-pub const MIGRATION_WEBSITE: &str = "https://halloy.chat/guides/migrating-from-yaml.html";
-pub const RELEASE_WEBSITE: &str = "https://github.com/squidowl/halloy/releases/latest";
+pub const MIGRATION_WEBSITE: &str =
+    "https://halloy.chat/guides/migrating-from-yaml.html";
+pub const RELEASE_WEBSITE: &str =
+    "https://github.com/squidowl/halloy/releases/latest";
 pub const SOURCE_WEBSITE: &str = "https://github.com/squidowl/halloy/";
 
 pub fn formatted_version() -> String {
@@ -51,6 +53,9 @@ fn portable_dir() -> Option<PathBuf> {
 fn platform_specific_config_dir() -> PathBuf {
     #[cfg(target_os = "macos")]
     {
+        // Priority order for config directory on macOS:
+        // 1. XDG config dir (~/.config/halloy)
+        // 2. User config directory (~/Library/Application Support/halloy)
         xdg_config_dir().unwrap_or_else(|| {
             dirs_next::config_dir()
                 .expect("expected valid config dir")
@@ -67,9 +72,12 @@ fn platform_specific_config_dir() -> PathBuf {
 
 #[cfg(target_os = "macos")]
 fn xdg_config_dir() -> Option<PathBuf> {
-    let config_dir = xdg::BaseDirectories::with_prefix("halloy")
-        .ok()
-        .and_then(|xdg| xdg.find_config_file(CONFIG_FILE_NAME))?;
+    let config_path = xdg::BaseDirectories::new().config_home?;
+    let halloy_config_dir = config_path.join("halloy");
 
-    config_dir.parent().map(|p| p.to_path_buf())
+    // if the config file exists, use the xdg config dir
+    halloy_config_dir
+        .join(CONFIG_FILE_NAME)
+        .is_file()
+        .then_some(halloy_config_dir)
 }
