@@ -1,4 +1,5 @@
 use std::cmp::Ordering;
+use std::collections::HashMap;
 use std::path::PathBuf;
 use std::time::Duration;
 use std::{fmt, io};
@@ -9,20 +10,23 @@ use futures::{Future, FutureExt};
 use tokio::fs;
 use tokio::time::Instant;
 
-use crate::message::{self, MessageReferences};
-use crate::target::{self, Target};
-use crate::{buffer, compression, environment, isupport, Buffer, Message, Server};
-
 pub use self::manager::{Manager, Resource};
 pub use self::metadata::{Metadata, ReadMarker};
+use crate::message::{self, MessageReferences, Source};
+use crate::target::{self, Target};
+use crate::user::Nick;
+use crate::{
+    Buffer, Message, Server, buffer, compression, environment, isupport,
+};
 
+pub mod filter;
 pub mod manager;
 pub mod metadata;
 
 // TODO: Make this configurable?
 /// Max # messages to persist
 const MAX_MESSAGES: usize = 10_000;
-/// # messages to tuncate after hitting [`MAX_MESSAGES`]
+/// # messages to truncate after hitting [`MAX_MESSAGES`]
 const TRUNC_COUNT: usize = 500;
 /// Duration to wait after receiving last message before flushing
 const FLUSH_AFTER_LAST_RECEIVED: Duration = Duration::from_secs(5);
@@ -60,22 +64,48 @@ impl Kind {
     pub fn from_input_buffer(buffer: buffer::Upstream) -> Self {
         match buffer {
             buffer::Upstream::Server(server) => Self::Server(server),
-            buffer::Upstream::Channel(server, channel) => Self::Channel(server, channel),
+            buffer::Upstream::Channel(server, channel) => {
+                Self::Channel(server, channel)
+            }
             buffer::Upstream::Query(server, nick) => Self::Query(server, nick),
         }
     }
 
-    pub fn from_server_message(server: Server, message: &Message) -> Option<Self> {
+    pub fn from_server_message(
+        server: Server,
+        message: &Message,
+    ) -> Option<Self> {
         match &message.target {
             message::Target::Server { .. } => Some(Self::Server(server)),
             message::Target::Channel { channel, .. } => {
                 Some(Self::Channel(server, channel.clone()))
             }
-            message::Target::Query { query, .. } => Some(Self::Query(server, query.clone())),
-            message::Target::Logs => None,
+            message::Target::Query { query, .. } => {
+                Some(Self::Query(server, query.clone()))
+            }
+            message::Target::Logs { .. } => None,
             message::Target::Highlights { .. } => None,
         }
     }
+
+    pub fn from_buffer(buffer: Buffer) -> Option<Self> {
+        match buffer {
+            Buffer::Upstream(buffer::Upstream::Server(server)) => {
+                Some(Kind::Server(server))
+            }
+            Buffer::Upstream(buffer::Upstream::Channel(server, channel)) => {
+                Some(Kind::Channel(server, channel))
+            }
+            Buffer::Upstream(buffer::Upstream::Query(server, nick)) => {
+                Some(Kind::Query(server, nick))
+            }
+            Buffer::Internal(buffer::Internal::Logs) => Some(Kind::Logs),
+            Buffer::Internal(buffer::Internal::Highlights) => {
+                Some(Kind::Highlights)
+            }
+            Buffer::Internal(buffer::Internal::FileTransfers) => None,
+        }
+    }
 }
 
 impl Kind {
@@ -89,11 +119,11 @@ impl Kind {
         }
     }
 
-    pub fn target(&self) -> Option<&str> {
+    pub fn target(&self) -> Option<Target> {
         match self {
             Kind::Server(_) => None,
-            Kind::Channel(_, channel) => Some(channel.as_str()),
-            Kind::Query(_, nick) => Some(nick.as_str()),
+            Kind::Channel(_, channel) => Some(Target::Channel(channel.clone())),
+            Kind::Query(_, nick) => Some(Target::Query(nick.clone())),
             Kind::Logs => None,
             Kind::Highlights => None,
         }
@@ -104,7 +134,9 @@ impl fmt::Display for Kind {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
             Kind::Server(server) => write!(f, "server on {server}"),
-            Kind::Channel(server, channel) => write!(f, "channel {channel} on {server}"),
+            Kind::Channel(server, channel) => {
+                write!(f, "channel {channel} on {server}")
+            }
             Kind::Query(server, nick) => write!(f, "user {nick} on {server}"),
             Kind::Logs => write!(f, "logs"),
             Kind::Highlights => write!(f, "highlights"),
@@ -115,11 +147,15 @@ impl fmt::Display for Kind {
 impl From<Kind> for Buffer {
     fn from(kind: Kind) -> Self {
         match kind {
-            Kind::Server(server) => Buffer::Upstream(buffer::Upstream::Server(server)),
+            Kind::Server(server) => {
+                Buffer::Upstream(buffer::Upstream::Server(server))
+            }
             Kind::Channel(server, channel) => {
                 Buffer::Upstream(buffer::Upstream::Channel(server, channel))
             }
-            Kind::Query(server, nick) => Buffer::Upstream(buffer::Upstream::Query(server, nick)),
+            Kind::Query(server, nick) => {
+                Buffer::Upstream(buffer::Upstream::Query(server, nick))
+            }
             Kind::Logs => Buffer::Internal(buffer::Internal::Logs),
             Kind::Highlights => Buffer::Internal(buffer::Internal::Highlights),
         }
@@ -198,9 +234,13 @@ async fn path(kind: &Kind) -> Result<PathBuf, Error> {
     let dir = dir_path().await?;
 
     let name = match kind {
-        Kind::Server(server) => format!("{server}"),
-        Kind::Channel(server, channel) => format!("{server}channel{}", channel.as_normalized_str()),
-        Kind::Query(server, query) => format!("{server}nickname{}", query.as_normalized_str()),
+        Kind::Server(server) => format!("{server:b}"),
+        Kind::Channel(server, channel) => {
+            format!("{server:b}channel{}", channel.as_normalized_str())
+        }
+        Kind::Query(server, query) => {
+            format!("{server:b}nickname{}", query.as_normalized_str())
+        }
         Kind::Logs => "logs".to_string(),
         Kind::Highlights => "highlights".to_string(),
     };
@@ -217,14 +257,18 @@ pub enum History {
         messages: Vec<Message>,
         last_updated_at: Option<Instant>,
         max_triggers_unread: Option<DateTime<Utc>>,
+        max_triggers_highlight: Option<DateTime<Utc>>,
         read_marker: Option<ReadMarker>,
         chathistory_references: Option<MessageReferences>,
+        last_seen: HashMap<Nick, DateTime<Utc>>,
     },
     Full {
         kind: Kind,
         messages: Vec<Message>,
         last_updated_at: Option<Instant>,
         read_marker: Option<ReadMarker>,
+        last_seen: HashMap<Nick, DateTime<Utc>>,
+        cleared: bool,
     },
 }
 
@@ -235,21 +279,27 @@ impl History {
             messages: vec![],
             last_updated_at: None,
             max_triggers_unread: None,
+            max_triggers_highlight: None,
             read_marker: None,
             chathistory_references: None,
+            last_seen: HashMap::new(),
         }
     }
 
     pub fn update_partial(&mut self, metadata: Metadata) {
         if let Self::Partial {
             max_triggers_unread,
+            max_triggers_highlight,
             read_marker,
             chathistory_references,
             ..
         } = self
         {
             *read_marker = (*read_marker).max(metadata.read_marker);
-            *max_triggers_unread = (*max_triggers_unread).max(metadata.last_triggers_unread);
+            *max_triggers_unread =
+                (*max_triggers_unread).max(metadata.last_triggers_unread);
+            *max_triggers_highlight =
+                (*max_triggers_highlight).max(metadata.last_triggers_highlight);
             *chathistory_references = chathistory_references
                 .clone()
                 .max(metadata.chathistory_references);
@@ -265,7 +315,8 @@ impl History {
             } => {
                 // Read marker is prior to last known message which triggers unread
                 if let Some(read_marker) = read_marker {
-                    max_triggers_unread.is_some_and(|max| read_marker.date_time() < max)
+                    max_triggers_unread
+                        .is_some_and(|max| read_marker.date_time() < max)
                 }
                 // Default state == unread if theres messages that trigger indicator
                 else {
@@ -276,36 +327,78 @@ impl History {
         }
     }
 
-    fn add_message(&mut self, message: Message) {
-        if message.triggers_unread() {
-            if let History::Partial {
+    fn has_highlight(&self) -> bool {
+        match self {
+            History::Partial {
+                max_triggers_highlight,
+                read_marker,
+                ..
+            } => {
+                // Read marker is prior to last known message which triggers highlight
+                if let Some(read_marker) = read_marker {
+                    max_triggers_highlight
+                        .is_some_and(|max| read_marker.date_time() < max)
+                }
+                // Default state == highlight if theres messages that trigger indicator
+                else {
+                    max_triggers_highlight.is_some()
+                }
+            }
+            History::Full { .. } => false,
+        }
+    }
+
+    fn add_message(&mut self, message: Message) -> Option<ReadMarker> {
+        if message.triggers_unread()
+            && !message.blocked
+            && let History::Partial {
                 max_triggers_unread,
                 ..
             } = self
-            {
-                *max_triggers_unread = (*max_triggers_unread).max(Some(message.server_time));
-            }
+        {
+            *max_triggers_unread =
+                (*max_triggers_unread).max(Some(message.server_time));
+        }
+
+        if message.triggers_highlight()
+            && !message.blocked
+            && let History::Partial {
+                max_triggers_highlight,
+                ..
+            } = self
+        {
+            *max_triggers_highlight =
+                (*max_triggers_highlight).max(Some(message.server_time));
         }
 
         match self {
             History::Partial {
                 messages,
                 last_updated_at,
+                last_seen,
                 ..
             }
             | History::Full {
                 messages,
                 last_updated_at,
+                last_seen,
                 ..
             } => {
                 *last_updated_at = Some(Instant::now());
 
-                insert_message(messages, message);
+                update_last_seen(last_seen, &message);
+
+                insert_message(messages, message)
             }
         }
     }
 
-    fn flush(&mut self, now: Instant) -> Option<BoxFuture<'static, Result<(), Error>>> {
+    // If now is None then history will be flushed regardless of time
+    // since last received
+    fn flush(
+        &mut self,
+        now: Option<Instant>,
+    ) -> Option<BoxFuture<'static, Result<(), Error>>> {
         match self {
             History::Partial {
                 kind,
@@ -314,20 +407,24 @@ impl History {
                 read_marker,
                 ..
             } => {
-                if let Some(last_received) = *last_updated_at {
-                    let since = now.duration_since(last_received);
+                if let Some(last_received) = *last_updated_at
+                    && now.is_none_or(|now| {
+                        now.duration_since(last_received)
+                            >= FLUSH_AFTER_LAST_RECEIVED
+                    })
+                {
+                    let kind = kind.clone();
+                    let messages = std::mem::take(messages);
+                    let read_marker = *read_marker;
 
-                    if since >= FLUSH_AFTER_LAST_RECEIVED {
-                        let kind = kind.clone();
-                        let messages = std::mem::take(messages);
-                        let read_marker = *read_marker;
+                    *last_updated_at = None;
 
-                        *last_updated_at = None;
-
-                        return Some(
-                            async move { append(&kind, messages, read_marker).await }.boxed(),
+                    return Some(
+                            async move {
+                                append(&kind, messages, read_marker).await
+                            }
+                            .boxed(),
                         );
-                    }
                 }
 
                 None
@@ -339,24 +436,31 @@ impl History {
                 read_marker,
                 ..
             } => {
-                if let Some(last_received) = *last_updated_at {
-                    let since = now.duration_since(last_received);
+                if let Some(last_received) = *last_updated_at
+                    && now.is_none_or(|now| {
+                        now.duration_since(last_received)
+                            >= FLUSH_AFTER_LAST_RECEIVED
+                    })
+                    && !messages.is_empty()
+                {
+                    let kind = kind.clone();
+                    let read_marker = *read_marker;
+                    *last_updated_at = None;
 
-                    if since >= FLUSH_AFTER_LAST_RECEIVED && !messages.is_empty() {
-                        let kind = kind.clone();
-                        let read_marker = *read_marker;
-                        *last_updated_at = None;
-
-                        if messages.len() > MAX_MESSAGES {
-                            messages.drain(0..messages.len() - (MAX_MESSAGES - TRUNC_COUNT));
-                        }
-
-                        let messages = messages.clone();
-
-                        return Some(
-                            async move { overwrite(&kind, &messages, read_marker).await }.boxed(),
+                    if messages.len() > MAX_MESSAGES {
+                        messages.drain(
+                            0..messages.len() - (MAX_MESSAGES - TRUNC_COUNT),
                         );
                     }
+
+                    let messages = messages.clone();
+
+                    return Some(
+                        async move {
+                            overwrite(&kind, &messages, read_marker).await
+                        }
+                        .boxed(),
+                    );
                 }
 
                 None
@@ -366,21 +470,26 @@ impl History {
 
     fn make_partial(
         &mut self,
-    ) -> Option<impl Future<Output = Result<Option<ReadMarker>, Error>> + use<>> {
+    ) -> Option<impl Future<Output = Result<(), Error>> + use<>> {
         match self {
             History::Partial { .. } => None,
             History::Full {
                 kind,
                 messages,
                 read_marker,
+                last_seen,
                 ..
             } => {
                 let kind = kind.clone();
                 let messages = std::mem::take(messages);
+                let read_marker = *read_marker;
+                let max_triggers_unread =
+                    metadata::latest_triggers_unread(&messages);
+                let max_triggers_highlight =
+                    metadata::latest_triggers_highlight(&messages);
 
-                let read_marker = ReadMarker::latest(&messages).max(*read_marker);
-                let max_triggers_unread = metadata::latest_triggers_unread(&messages);
-                let chathistory_references = metadata::latest_can_reference(&messages);
+                let chathistory_references =
+                    metadata::latest_can_reference(&messages);
 
                 *self = Self::Partial {
                     kind: kind.clone(),
@@ -388,48 +497,82 @@ impl History {
                     last_updated_at: None,
                     read_marker,
                     max_triggers_unread,
+                    max_triggers_highlight,
                     chathistory_references,
+                    last_seen: last_seen.clone(),
                 };
 
-                Some(async move {
-                    overwrite(&kind, &messages, read_marker)
-                        .await
-                        .map(|_| read_marker)
-                })
+                Some(
+                    async move { overwrite(&kind, &messages, read_marker).await },
+                )
             }
         }
     }
 
-    async fn close(self) -> Result<Option<ReadMarker>, Error> {
+    async fn close(self) -> Result<(), Error> {
         match self {
             History::Partial {
                 kind,
                 messages,
                 read_marker,
                 ..
-            } => {
-                append(&kind, messages, read_marker).await?;
-
-                Ok(None)
-            }
+            } => append(&kind, messages, read_marker).await,
             History::Full {
                 kind,
                 messages,
                 read_marker,
                 ..
+            } => overwrite(&kind, &messages, read_marker).await,
+        }
+    }
+
+    pub fn mark_as_read(&mut self) -> Option<ReadMarker> {
+        let (read_marker, latest) = match self {
+            History::Partial {
+                max_triggers_unread,
+                read_marker,
+                ..
+            } => (
+                read_marker,
+                max_triggers_unread.map(ReadMarker::from_date_time),
+            ),
+            History::Full {
+                messages,
+                read_marker,
+                ..
+            } => (read_marker, ReadMarker::latest(messages)),
+        };
+
+        if latest > *read_marker {
+            *read_marker = latest;
+
+            latest
+        } else {
+            None
+        }
+    }
+
+    pub fn can_mark_as_read(&self) -> bool {
+        match self {
+            History::Partial { .. } => self.has_unread(),
+            History::Full {
+                messages,
+                read_marker,
+                ..
             } => {
-                let read_marker = ReadMarker::latest(&messages).max(read_marker);
-
-                overwrite(&kind, &messages, read_marker).await?;
-
-                Ok(read_marker)
+                if messages.is_empty() {
+                    false
+                } else {
+                    *read_marker < ReadMarker::latest(messages)
+                }
             }
         }
     }
 
     pub fn first_can_reference(&self) -> Option<&Message> {
         match self {
-            History::Partial { messages, .. } | History::Full { messages, .. } => {
+            History::Partial { messages, .. }
+            | History::Full { messages, .. } => {
                 messages.iter().find(|message| message.can_reference())
             }
         }
@@ -447,14 +590,15 @@ impl History {
             } => messages
                 .iter()
                 .rev()
-                .find(|message| message.can_reference() && message.server_time < server_time)
+                .find(|message| {
+                    message.can_reference() && message.server_time < server_time
+                })
                 .map_or(
-                    if chathistory_references
-                        .as_ref()
-                        .is_some_and(|chathistory_references| {
+                    if chathistory_references.as_ref().is_some_and(
+                        |chathistory_references| {
                             chathistory_references.timestamp < server_time
-                        })
-                    {
+                        },
+                    ) {
                         chathistory_references.clone()
                     } else {
                         None
@@ -464,8 +608,10 @@ impl History {
             History::Full { messages, .. } => messages
                 .iter()
                 .rev()
-                .find(|message| message.can_reference() && message.server_time < server_time)
-                .map(|message| message.references()),
+                .find(|message| {
+                    message.can_reference() && message.server_time < server_time
+                })
+                .map(Message::references),
         }
     }
 
@@ -480,9 +626,8 @@ impl History {
 
     pub fn read_marker(&self) -> Option<ReadMarker> {
         match self {
-            History::Partial { read_marker, .. } | History::Full { read_marker, .. } => {
-                *read_marker
-            }
+            History::Partial { read_marker, .. }
+            | History::Full { read_marker, .. } => *read_marker,
         }
     }
 
@@ -492,12 +637,19 @@ impl History {
             last_updated_at,
             ..
         } = self
+            && let Some(message) =
+                messages.iter_mut().find(|m| m.hash == message)
         {
-            if let Some(message) = messages.iter_mut().find(|m| m.hash == message) {
-                message.hidden_urls.insert(url);
+            message.hidden_urls.insert(url);
 
-                *last_updated_at = Some(Instant::now());
-            }
+            *last_updated_at = Some(Instant::now());
+        }
+    }
+
+    pub fn last_seen(&self) -> HashMap<Nick, DateTime<Utc>> {
+        match self {
+            History::Partial { last_seen, .. }
+            | History::Full { last_seen, .. } => last_seen.clone(),
         }
     }
 }
@@ -508,9 +660,17 @@ impl History {
 /// Deduplication is only checked +/- 1 second around the server time
 /// of the incoming message. Either message IDs match, or server times
 /// have an exact match + target & content.
-pub fn insert_message(messages: &mut Vec<Message>, message: Message) {
+///
+/// A non-None return value indicates whether a message sent from
+/// this client was replaced by an echo
+pub fn insert_message(
+    messages: &mut Vec<Message>,
+    message: Message,
+) -> Option<ReadMarker> {
     let fuzz_seconds =
-        if matches!(message.direction, message::Direction::Received) && message.is_echo {
+        if matches!(message.direction, message::Direction::Received)
+            && message.is_echo
+        {
             chrono::Duration::seconds(300)
         } else {
             chrono::Duration::seconds(1)
@@ -519,17 +679,21 @@ pub fn insert_message(messages: &mut Vec<Message>, message: Message) {
     if messages.is_empty() {
         messages.push(message);
 
-        return;
+        return None;
     }
 
     let start = message.server_time - fuzz_seconds;
     let end = message.server_time + fuzz_seconds;
 
-    let start_index = match messages.binary_search_by(|stored| stored.server_time.cmp(&start)) {
+    let start_index = match messages
+        .binary_search_by(|stored| stored.server_time.cmp(&start))
+    {
         Ok(match_index) => match_index,
         Err(sorted_insert_index) => sorted_insert_index,
     };
-    let end_index = match messages.binary_search_by(|stored| stored.server_time.cmp(&end)) {
+    let end_index = match messages
+        .binary_search_by(|stored| stored.server_time.cmp(&end))
+    {
         Ok(match_index) => match_index,
         Err(sorted_insert_index) => sorted_insert_index,
     };
@@ -539,15 +703,21 @@ pub fn insert_message(messages: &mut Vec<Message>, message: Message) {
     let mut replace_at = None;
 
     for stored in &messages[start_index..end_index] {
-        if replace_at.is_none()
-            && ((message.id.is_some() && stored.id == message.id)
-                || ((stored.server_time == message.server_time
-                    || (matches!(stored.direction, message::Direction::Sent)
-                        && matches!(message.direction, message::Direction::Received)
-                        && message.is_echo))
-                    && has_matching_content(stored, &message)))
-        {
-            replace_at = Some(current_index);
+        if replace_at.is_none() {
+            let use_echo_cmp =
+                matches!(stored.direction, message::Direction::Sent)
+                    && matches!(
+                        message.direction,
+                        message::Direction::Received
+                    )
+                    && message.is_echo;
+
+            if (message.id.is_some() && stored.id == message.id)
+                || ((stored.server_time == message.server_time || use_echo_cmp)
+                    && has_matching_content(stored, &message, use_echo_cmp))
+            {
+                replace_at = Some(current_index);
+            }
         }
 
         if message.server_time >= stored.server_time {
@@ -559,13 +729,28 @@ pub fn insert_message(messages: &mut Vec<Message>, message: Message) {
 
     if let Some(index) = replace_at {
         if messages[index].server_time == message.server_time {
-            if has_matching_content(&messages[index], &message) {
+            if has_matching_content(&messages[index], &message, false) {
                 messages[index].id = message.id;
                 messages[index].received_at = message.received_at;
             } else {
                 messages[index] = message;
             }
+
+            None
         } else {
+            let read_marker = if matches!(
+                messages[index].direction,
+                message::Direction::Sent
+            ) && matches!(
+                message.direction,
+                message::Direction::Received
+            ) && message.is_echo
+            {
+                Some(ReadMarker::from_date_time(message.server_time))
+            } else {
+                None
+            };
+
             match insert_at.cmp(&index) {
                 Ordering::Less => {
                     messages.remove(index);
@@ -577,16 +762,24 @@ pub fn insert_message(messages: &mut Vec<Message>, message: Message) {
                     messages.remove(index);
                 }
             }
+
+            read_marker
         }
     } else {
         messages.insert(insert_at, message);
+
+        None
     }
 }
 
 /// The content of JOIN, PART, and QUIT messages may be dependent on how
 /// the user attributes are resolved.  Match those messages based on Nick
 /// alone (covered by comparing target components) to avoid false negatives.
-fn has_matching_content(message: &Message, other: &Message) -> bool {
+fn has_matching_content(
+    message: &Message,
+    other: &Message,
+    use_echo_cmp: bool,
+) -> bool {
     if message.target == other.target {
         if let message::Source::Server(Some(source)) = message.target.source() {
             match source.kind() {
@@ -597,18 +790,54 @@ fn has_matching_content(message: &Message, other: &Message) -> bool {
                 }
                 message::source::server::Kind::ReplyTopic
                 | message::source::server::Kind::ChangeHost
+                | message::source::server::Kind::ChangeNick
+                | message::source::server::Kind::ChangeMode
+                | message::source::server::Kind::ChangeTopic
                 | message::source::server::Kind::MonitoredOnline
                 | message::source::server::Kind::MonitoredOffline
-                | message::source::server::Kind::StandardReply(_) => (),
+                | message::source::server::Kind::StandardReply(_)
+                | message::source::server::Kind::WAllOps
+                | message::source::server::Kind::Kick => (),
             }
         }
 
-        message.content == other.content
+        if use_echo_cmp {
+            matches!(message.content.echo_cmp(&other.content), Ordering::Equal)
+        } else {
+            message.content == other.content
+        }
     } else {
         false
     }
 }
 
+pub fn update_last_seen(
+    last_seen: &mut HashMap<Nick, DateTime<Utc>>,
+    message: &Message,
+) {
+    if let Source::User(user) = message.target.source() {
+        let nickname = user.nickname().to_owned();
+
+        if let Some(date_time) = last_seen.get_mut(&nickname) {
+            if message.server_time > *date_time {
+                *date_time = message.server_time;
+            }
+        } else {
+            last_seen.insert(nickname, message.server_time);
+        }
+    }
+}
+
+pub fn get_last_seen(messages: &[Message]) -> HashMap<Nick, DateTime<Utc>> {
+    let mut last_seen = HashMap::new();
+
+    messages.iter().for_each(|message| {
+        update_last_seen(&mut last_seen, message);
+    });
+
+    last_seen
+}
+
 #[derive(Debug)]
 pub struct View<'a> {
     pub total: usize,
@@ -618,6 +847,7 @@ pub struct View<'a> {
     pub new_messages: Vec<&'a Message>,
     pub max_nick_chars: Option<usize>,
     pub max_prefix_chars: Option<usize>,
+    pub cleared: bool,
 }
 
 #[derive(Debug, thiserror::Error)]
