@@ -2,36 +2,56 @@ use std::fmt;
 use std::path::PathBuf;
 use std::str::FromStr;
 
-use chrono::{format::SecondsFormat, DateTime, Utc};
+use chrono::format::SecondsFormat;
+use chrono::{DateTime, Utc};
 use serde::{Deserialize, Serialize};
 use tokio::fs;
 
-use crate::history::{dir_path, Error, Kind};
-use crate::message::{source, MessageReferences};
 use crate::Message;
+use crate::history::{Error, Kind, dir_path};
+use crate::message::{MessageReferences, source};
 
 #[derive(Debug, Clone, Default, Deserialize, Serialize)]
 pub struct Metadata {
     pub read_marker: Option<ReadMarker>,
     pub last_triggers_unread: Option<DateTime<Utc>>,
+    pub last_triggers_highlight: Option<DateTime<Utc>>,
     pub chathistory_references: Option<MessageReferences>,
 }
 
-#[derive(Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Default, Deserialize, Serialize)]
+#[derive(
+    Debug,
+    Clone,
+    Copy,
+    PartialEq,
+    Eq,
+    PartialOrd,
+    Ord,
+    Default,
+    Deserialize,
+    Serialize,
+)]
 pub struct ReadMarker(DateTime<Utc>);
 
 impl ReadMarker {
+    pub fn from_date_time(date_time: DateTime<Utc>) -> Self {
+        ReadMarker(date_time)
+    }
+
     pub fn latest(messages: &[Message]) -> Option<Self> {
         messages
             .iter()
             .rev()
-            .find(|message| match message.target.source() {
-                source::Source::Internal(source) => match source {
-                    source::Internal::Status(_) => false,
-                    // Logs are in their own buffer and this gives us backlog support there
-                    source::Internal::Logs => true,
-                },
-                _ => true,
+            .find(|message| {
+                !message.blocked
+                    && match message.target.source() {
+                        source::Source::Internal(source) => match source {
+                            source::Internal::Status(_) => false,
+                            // Logs are in their own buffer and this gives us backlog support there
+                            source::Internal::Logs(_) => true,
+                        },
+                        _ => true,
+                    }
             })
             .map(|message| message.server_time)
             .map(Self)
@@ -66,12 +86,22 @@ pub fn latest_triggers_unread(messages: &[Message]) -> Option<DateTime<Utc>> {
         .map(|message| message.server_time)
 }
 
+pub fn latest_triggers_highlight(
+    messages: &[Message],
+) -> Option<DateTime<Utc>> {
+    messages
+        .iter()
+        .rev()
+        .find(|message| message.triggers_highlight())
+        .map(|message| message.server_time)
+}
+
 pub fn latest_can_reference(messages: &[Message]) -> Option<MessageReferences> {
     messages
         .iter()
         .rev()
         .find(|message| message.can_reference())
-        .map(|message| message.references())
+        .map(Message::references)
 }
 
 pub async fn load(kind: Kind) -> Result<Metadata, Error> {
@@ -92,6 +122,7 @@ pub async fn save(
     let bytes = serde_json::to_vec(&Metadata {
         read_marker,
         last_triggers_unread: latest_triggers_unread(messages),
+        last_triggers_highlight: latest_triggers_highlight(messages),
         chathistory_references: latest_can_reference(messages),
     })?;
 
@@ -102,19 +133,22 @@ pub async fn save(
     Ok(())
 }
 
-pub async fn update(kind: &Kind, read_marker: &ReadMarker) -> Result<(), Error> {
+pub async fn update(
+    kind: &Kind,
+    read_marker: &ReadMarker,
+) -> Result<(), Error> {
     let metadata = load(kind.clone()).await?;
 
-    if metadata
-        .read_marker
-        .is_some_and(|metadata_read_marker| metadata_read_marker >= *read_marker)
-    {
+    if metadata.read_marker.is_some_and(|metadata_read_marker| {
+        metadata_read_marker >= *read_marker
+    }) {
         return Ok(());
     }
 
     let bytes = serde_json::to_vec(&Metadata {
         read_marker: Some(*read_marker),
         last_triggers_unread: metadata.last_triggers_unread,
+        last_triggers_highlight: metadata.last_triggers_highlight,
         chathistory_references: metadata.chathistory_references,
     })?;
 
