@@ -1,22 +1,24 @@
-use std::{collections::HashSet, mem};
+use std::collections::HashSet;
+use std::mem;
 
 use iced_core::color;
 use itertools::PeekingNext;
 use serde::{Deserialize, Serialize};
 
-use crate::appearance::theme;
-
 pub use self::encode::encode;
+use crate::appearance::theme;
 
 pub mod encode;
 
-pub fn parse(text: &str) -> Option<Vec<Fragment>> {
+pub fn parse(
+    text: &str,
+    modifiers: &mut HashSet<Modifier>,
+    fg: &mut Option<Color>,
+    bg: &mut Option<Color>,
+) -> Option<Vec<Fragment>> {
     let mut fragments = vec![];
 
     let mut current_text = String::new();
-    let mut modifiers = HashSet::new();
-    let mut fg = None;
-    let mut bg = None;
 
     let mut iter = text.chars().peekable();
 
@@ -30,7 +32,7 @@ pub fn parse(text: &str) -> Option<Vec<Fragment>> {
                 } else {
                     fragments.push(Fragment::Formatted(
                         text,
-                        Formatting::new(&modifiers, fg, bg),
+                        Formatting::new(modifiers, *fg, *bg),
                     ));
                 }
             }
@@ -38,34 +40,39 @@ pub fn parse(text: &str) -> Option<Vec<Fragment>> {
             match modifier {
                 Modifier::Reset => {
                     modifiers.clear();
-                    fg = None;
-                    bg = None;
+                    *fg = None;
+                    *bg = None;
                 }
                 Modifier::Color => {
                     // Trailing digit for new color, otherwise resets
                     if let Some(c) = iter.peeking_next(char::is_ascii_digit) {
                         // 1-2 digiits
                         let mut digits = c.to_string();
-                        if let Some(c) = iter.peeking_next(char::is_ascii_digit) {
+                        if let Some(c) = iter.peeking_next(char::is_ascii_digit)
+                        {
                             digits.push(c);
                         }
 
                         let code = digits.parse().ok()?;
 
-                        fg = Color::code(code);
+                        *fg = Color::code(code);
 
                         if let Some(comma) = iter.peeking_next(|c| *c == ',') {
                             // Has background
-                            if let Some(c) = iter.peeking_next(char::is_ascii_digit) {
+                            if let Some(c) =
+                                iter.peeking_next(char::is_ascii_digit)
+                            {
                                 // 1-2 digits
                                 let mut digits = c.to_string();
-                                if let Some(c) = iter.peeking_next(char::is_ascii_digit) {
+                                if let Some(c) =
+                                    iter.peeking_next(char::is_ascii_digit)
+                                {
                                     digits.push(c);
                                 }
 
                                 let code = digits.parse().ok()?;
 
-                                bg = Color::code(code);
+                                *bg = Color::code(code);
                             }
                             // Nope, just a normal char
                             else {
@@ -73,39 +80,72 @@ pub fn parse(text: &str) -> Option<Vec<Fragment>> {
                             }
                         }
                     } else {
-                        fg = None;
-                        bg = None;
+                        *fg = None;
+                        *bg = None;
                     }
                 }
                 Modifier::HexColor => {
                     // Trailing digit for new color, otherwise resets
-                    if let Some(c) = iter.peeking_next(char::is_ascii_hexdigit) {
+                    if let Some(c) = iter.peeking_next(char::is_ascii_hexdigit)
+                    {
                         // 6 digits (hex)
-                        let mut hex = c.to_string();
+                        let mut hex = Vec::from([c]);
                         for _ in 0..5 {
                             hex.push(iter.next()?);
                         }
 
-                        let r = u8::from_str_radix(&hex[0..2], 16).ok()?;
-                        let g = u8::from_str_radix(&hex[2..4], 16).ok()?;
-                        let b = u8::from_str_radix(&hex[4..6], 16).ok()?;
+                        let r = u8::from_str_radix(
+                            &hex.iter().take(2).collect::<String>(),
+                            16,
+                        )
+                        .ok()?;
+                        let g = u8::from_str_radix(
+                            &hex.iter().skip(2).take(2).collect::<String>(),
+                            16,
+                        )
+                        .ok()?;
+                        let b = u8::from_str_radix(
+                            &hex.iter().skip(4).take(2).collect::<String>(),
+                            16,
+                        )
+                        .ok()?;
 
-                        fg = Some(Color::Rgb(r, g, b));
+                        *fg = Some(Color::Rgb(r, g, b));
 
                         if let Some(comma) = iter.peeking_next(|c| *c == ',') {
                             // Has background
-                            if let Some(c) = iter.peeking_next(char::is_ascii_hexdigit) {
+                            if let Some(c) =
+                                iter.peeking_next(char::is_ascii_hexdigit)
+                            {
                                 // 6 digits (hex)
-                                let mut hex = c.to_string();
+                                let mut hex = Vec::from([c]);
                                 for _ in 0..5 {
                                     hex.push(iter.next()?);
                                 }
 
-                                let r = u8::from_str_radix(&hex[0..2], 16).ok()?;
-                                let g = u8::from_str_radix(&hex[2..4], 16).ok()?;
-                                let b = u8::from_str_radix(&hex[4..6], 16).ok()?;
+                                let r = u8::from_str_radix(
+                                    &hex.iter().take(2).collect::<String>(),
+                                    16,
+                                )
+                                .ok()?;
+                                let g = u8::from_str_radix(
+                                    &hex.iter()
+                                        .skip(2)
+                                        .take(2)
+                                        .collect::<String>(),
+                                    16,
+                                )
+                                .ok()?;
+                                let b = u8::from_str_radix(
+                                    &hex.iter()
+                                        .skip(4)
+                                        .take(2)
+                                        .collect::<String>(),
+                                    16,
+                                )
+                                .ok()?;
 
-                                bg = Some(Color::Rgb(r, g, b));
+                                *bg = Some(Color::Rgb(r, g, b));
                             }
                             // Nope, just a normal char
                             else {
@@ -113,8 +153,8 @@ pub fn parse(text: &str) -> Option<Vec<Fragment>> {
                             }
                         }
                     } else {
-                        fg = None;
-                        bg = None;
+                        *fg = None;
+                        *bg = None;
                     }
                 }
                 m => {
@@ -138,21 +178,19 @@ pub fn parse(text: &str) -> Option<Vec<Fragment>> {
         } else {
             fragments.push(Fragment::Formatted(
                 text,
-                Formatting::new(&modifiers, fg, bg),
+                Formatting::new(modifiers, *fg, *bg),
             ));
         }
     }
 
-    if fragments.is_empty()
-        || (fragments.len() == 1 && matches!(fragments.first(), Some(Fragment::Unformatted(_))))
-    {
-        None
-    } else {
-        Some(fragments)
-    }
+    // Only return None if parsing failed; return an empty fragments if there
+    // are no non-formatting characters after parsing
+    Some(fragments)
 }
 
-#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash, Serialize, Deserialize)]
+#[derive(
+    Debug, Default, Clone, Copy, PartialEq, Eq, Hash, Serialize, Deserialize,
+)]
 pub struct Formatting {
     pub bold: bool,
     pub italics: bool,
@@ -164,7 +202,11 @@ pub struct Formatting {
 }
 
 impl Formatting {
-    fn new(modifiers: &HashSet<Modifier>, fg: Option<Color>, bg: Option<Color>) -> Self {
+    fn new(
+        modifiers: &HashSet<Modifier>,
+        fg: Option<Color>,
+        bg: Option<Color>,
+    ) -> Self {
         let (fg, bg) = if modifiers.contains(&Modifier::ReverseColor) {
             (bg, fg)
         } else {
@@ -191,7 +233,7 @@ pub enum Fragment {
 
 #[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
 #[repr(u8)]
-enum Modifier {
+pub enum Modifier {
     Bold = 0x02,
     Italics = 0x1D,
     Underline = 0x1F,
@@ -553,7 +595,10 @@ impl Color {
         }
     }
 
-    pub fn into_iced(self, _colors: &theme::Colors) -> Option<iced_core::Color> {
+    pub fn into_iced(
+        self,
+        _styles: &theme::Styles,
+    ) -> Option<iced_core::Color> {
         // TODO: Theme aware 0 - 15 colors
         match self {
             Color::White => Some(color!(0xffffff)),
