@@ -1,11 +1,12 @@
-use chrono::{DateTime, Utc};
-use futures::never::Never;
+use std::sync::Arc;
 use std::time::Duration;
 
+use chrono::{DateTime, Utc};
 use futures::channel::mpsc;
-use futures::{future, stream, FutureExt, SinkExt, StreamExt};
-use irc::proto::{self, command, Command};
-use irc::{codec, connection, Connection};
+use futures::never::Never;
+use futures::{FutureExt, SinkExt, StreamExt, future, stream};
+use irc::proto::{self, Command, command};
+use irc::{Connection, codec, connection};
 use tokio::time::{self, Instant, Interval};
 
 use crate::client::Client;
@@ -13,6 +14,8 @@ use crate::server::Server;
 use crate::time::Posix;
 use crate::{config, message, server};
 
+const QUIT_REQUEST_TIMEOUT: Duration = Duration::from_millis(400);
+
 pub type Result<T = Update, E = Error> = std::result::Result<T, E>;
 
 #[derive(Debug)]
@@ -52,6 +55,7 @@ enum State {
         batch: Batch,
         ping_time: Interval,
         ping_timeout: Option<Interval>,
+        quit_requested: Option<(Instant, Option<String>)>,
     },
     Quit,
 }
@@ -62,6 +66,7 @@ enum Input {
     Send(proto::Message),
     Ping,
     PingTimeout,
+    Quit(Option<String>),
 }
 
 struct Stream {
@@ -76,8 +81,9 @@ pub fn run(
     let (sender, receiver) = mpsc::unbounded();
 
     // Spawn to unblock backend from iced stream which has backpressure
-    let runner = stream::once(async { tokio::spawn(_run(server, proxy, sender)).await })
-        .map(|_| unreachable!());
+    let runner =
+        stream::once(async { tokio::spawn(_run(server, proxy, sender)).await })
+            .map(|_| unreachable!());
 
     stream::select(receiver, runner)
 }
@@ -106,14 +112,17 @@ async fn _run(
         match &mut state {
             State::Disconnected { last_retry } => {
                 if let Some(last_retry) = last_retry.as_ref() {
-                    let remaining = reconnect_delay.saturating_sub(last_retry.elapsed());
+                    let remaining =
+                        reconnect_delay.saturating_sub(last_retry.elapsed());
 
                     if !remaining.is_zero() {
                         time::sleep(remaining).await;
                     }
                 }
 
-                match connect(server.clone(), config.clone(), proxy.clone()).await {
+                match connect(server.clone(), config.clone(), proxy.clone())
+                    .await
+                {
                     Ok((stream, client)) => {
                         log::info!("[{server}] connected");
 
@@ -131,22 +140,26 @@ async fn _run(
                             batch: Batch::new(),
                             ping_timeout: None,
                             ping_time: ping_time_interval(config.ping_time),
+                            quit_requested: None,
                         };
                     }
                     Err(e) => {
                         let error = match e {
                             // unwrap Tls-specific error enums to access more error info
-                            connection::Error::Tls(e) => format!("a TLS error occured: {e}"),
+                            connection::Error::Tls(e) => {
+                                format!("a TLS error occurred: {e}")
+                            }
                             _ => e.to_string(),
                         };
 
-                        log::warn!("[{server}] connection failed: {error}");
+                        log::info!("[{server}] connection failed: {error}");
 
-                        let _ = sender.unbounded_send(Update::ConnectionFailed {
-                            server: server.clone(),
-                            error,
-                            sent_time: Utc::now(),
-                        });
+                        let _ =
+                            sender.unbounded_send(Update::ConnectionFailed {
+                                server: server.clone(),
+                                error,
+                                sent_time: Utc::now(),
+                            });
 
                         *last_retry = Some(Instant::now());
                     }
@@ -157,12 +170,17 @@ async fn _run(
                 batch,
                 ping_time,
                 ping_timeout,
+                quit_requested,
             } => {
                 let input = {
                     let mut select = stream::select_all([
                         (&mut stream.connection).map(Input::IrcMessage).boxed(),
                         (&mut stream.receiver).map(Input::Send).boxed(),
-                        ping_time.tick().into_stream().map(|_| Input::Ping).boxed(),
+                        ping_time
+                            .tick()
+                            .into_stream()
+                            .map(|_| Input::Ping)
+                            .boxed(),
                         batch.map(Input::Batch).boxed(),
                     ]);
 
@@ -176,13 +194,28 @@ async fn _run(
                         );
                     }
 
+                    if let Some((requested_at, reason)) = quit_requested {
+                        select.push(
+                            time::sleep_until(
+                                *requested_at + QUIT_REQUEST_TIMEOUT,
+                            )
+                            .into_stream()
+                            .map(|()| Input::Quit(reason.clone()))
+                            .boxed(),
+                        );
+                    }
+
                     select.next().await.expect("stream input")
                 };
 
                 match input {
-                    Input::IrcMessage(Ok(Ok(message))) => match message.command {
+                    Input::IrcMessage(Ok(Ok(message))) => match message.command
+                    {
                         proto::Command::PING(token) => {
-                            let _ = stream.connection.send(command!("PONG", token)).await;
+                            let _ = stream
+                                .connection
+                                .send(command!("PONG", token))
+                                .await;
                         }
                         proto::Command::PONG(_, token) => {
                             let token = token.unwrap_or_default();
@@ -191,16 +224,33 @@ async fn _run(
                             *ping_timeout = None;
                         }
                         proto::Command::ERROR(error) => {
-                            log::warn!("[{server}] disconnected: {error}");
-                            let _ = sender.unbounded_send(Update::Disconnected {
-                                server: server.clone(),
-                                is_initial,
-                                error: Some(error),
-                                sent_time: Utc::now(),
-                            });
-                            state = State::Disconnected {
-                                last_retry: Some(Instant::now()),
-                            };
+                            if let Some(reason) = quit_requested
+                                .as_ref()
+                                .map(|(_, reason)| reason)
+                            {
+                                // If QUIT was requested, then ERROR is
+                                // a valid acknowledgement
+                                // https://modern.ircdocs.horse/#quit-message
+                                let _ = sender.unbounded_send(Update::Quit(
+                                    server.clone(),
+                                    reason.clone(),
+                                ));
+
+                                state = State::Quit;
+                            } else {
+                                log::info!("[{server}] disconnected: {error}");
+                                let _ = sender.unbounded_send(
+                                    Update::Disconnected {
+                                        server: server.clone(),
+                                        is_initial,
+                                        error: Some(error),
+                                        sent_time: Utc::now(),
+                                    },
+                                );
+                                state = State::Disconnected {
+                                    last_retry: Some(Instant::now()),
+                                };
+                            }
                         }
                         _ => {
                             batch.messages.push(message.into());
@@ -210,7 +260,7 @@ async fn _run(
                         log::warn!("message decoding failed: {e}");
                     }
                     Input::IrcMessage(Err(e)) => {
-                        log::warn!("[{server}] disconnected: {e}");
+                        log::info!("[{server}] disconnected: {e}");
                         let _ = sender.unbounded_send(Update::Disconnected {
                             server: server.clone(),
                             is_initial,
@@ -222,21 +272,23 @@ async fn _run(
                         };
                     }
                     Input::Batch(messages) => {
-                        let _ = sender
-                            .unbounded_send(Update::MessagesReceived(server.clone(), messages));
+                        let _ = sender.unbounded_send(
+                            Update::MessagesReceived(server.clone(), messages),
+                        );
                     }
                     Input::Send(message) => {
-                        log::trace!("[{server}] Sending message => {:?}", message);
+                        log::trace!(
+                            "[{server}] Sending message => {message:?}"
+                        );
 
                         if let Command::QUIT(reason) = &message.command {
                             let reason = reason.clone();
 
                             let _ = stream.connection.send(message).await;
-                            let _ = sender.unbounded_send(Update::Quit(server.clone(), reason));
 
                             log::info!("[{server}] quit");
 
-                            state = State::Quit;
+                            *quit_requested = Some((Instant::now(), reason));
                         } else {
                             let _ = stream.connection.send(message).await;
                         }
@@ -245,14 +297,17 @@ async fn _run(
                         let now = Posix::now().as_nanos().to_string();
                         log::trace!("[{server}] ping sent: {now}");
 
-                        let _ = stream.connection.send(command!("PING", now)).await;
+                        let _ =
+                            stream.connection.send(command!("PING", now)).await;
 
                         if ping_timeout.is_none() {
-                            *ping_timeout = Some(ping_timeout_interval(config.ping_timeout));
+                            *ping_timeout = Some(ping_timeout_interval(
+                                config.ping_timeout,
+                            ));
                         }
                     }
                     Input::PingTimeout => {
-                        log::warn!("[{server}] ping timeout");
+                        log::info!("[{server}] ping timeout");
                         let _ = sender.unbounded_send(Update::Disconnected {
                             server: server.clone(),
                             is_initial,
@@ -263,6 +318,14 @@ async fn _run(
                             last_retry: Some(Instant::now()),
                         };
                     }
+                    Input::Quit(reason) => {
+                        let _ = sender.unbounded_send(Update::Quit(
+                            server.clone(),
+                            reason,
+                        ));
+
+                        state = State::Quit;
+                    }
                 }
             }
             State::Quit => {
@@ -275,16 +338,17 @@ async fn _run(
 
 async fn connect(
     server: Server,
-    config: config::Server,
+    config: Arc<config::Server>,
     proxy: Option<config::Proxy>,
 ) -> Result<(Stream, Client), connection::Error> {
-    let connection = Connection::new(config.connection(proxy), irc::Codec).await?;
+    let connection =
+        Connection::new(config.connection(proxy), irc::Codec).await?;
 
     let (sender, receiver) = mpsc::channel(100);
 
     let mut client = Client::new(server, config, sender);
     if let Err(e) = client.connect() {
-        log::error!("Error when connecting client: {:?}", e);
+        log::error!("Error when connecting client: {e:?}");
     }
 
     Ok((
