@@ -1,5 +1,6 @@
 use std::io;
 use std::path::PathBuf;
+use std::sync::Arc;
 
 use iced_core::{Point, Size};
 use serde::{Deserialize, Serialize};
@@ -13,10 +14,11 @@ pub mod position;
 pub mod size;
 
 #[derive(Debug, Clone, Copy, Serialize, Deserialize)]
+#[serde(default)]
 pub struct Window {
-    #[serde(default, with = "serde_position")]
+    #[serde(with = "serde_position")]
     pub position: Option<Point>,
-    #[serde(default = "default_size", with = "serde_size")]
+    #[serde(with = "serde_size")]
     pub size: Size,
 }
 
@@ -24,18 +26,14 @@ impl Default for Window {
     fn default() -> Self {
         Self {
             position: None,
-            size: default_size(),
+            size: Size {
+                width: 1024.0,
+                height: 768.0,
+            },
         }
     }
 }
 
-pub fn default_size() -> Size {
-    Size {
-        width: 1024.0,
-        height: 768.0,
-    }
-}
-
 impl Window {
     pub async fn load() -> Result<Window, Error> {
         let path = path()?;
@@ -43,7 +41,9 @@ impl Window {
         let Window { position, size } = serde_json::from_slice(&bytes)?;
 
         let size = size.max(MIN_SIZE);
-        let position = position.filter(|pos| pos.y.is_sign_positive() && pos.x.is_sign_positive());
+        let position = position
+            .filter(|pos| pos.y.is_sign_positive() && pos.x.is_sign_positive())
+            .filter(|pos| is_position_valid(*pos));
 
         Ok(Window { position, size })
     }
@@ -68,12 +68,29 @@ fn path() -> Result<PathBuf, Error> {
     Ok(parent.join("window.json"))
 }
 
-#[derive(Debug, thiserror::Error)]
+/// Check if a window position is valid (within visible screen bounds)
+fn is_position_valid(position: Point) -> bool {
+    display_info::DisplayInfo::from_point(position.x as i32, position.y as i32).is_ok()
+}
+
+#[derive(Debug, Clone, thiserror::Error)]
 pub enum Error {
     #[error(transparent)]
-    Serde(#[from] serde_json::Error),
+    Serde(Arc<serde_json::Error>),
     #[error(transparent)]
-    Io(#[from] io::Error),
+    Io(Arc<io::Error>),
+}
+
+impl From<serde_json::Error> for Error {
+    fn from(error: serde_json::Error) -> Self {
+        Self::Serde(Arc::new(error))
+    }
+}
+
+impl From<io::Error> for Error {
+    fn from(error: io::Error) -> Self {
+        Self::Io(Arc::new(error))
+    }
 }
 
 mod serde_position {
@@ -87,7 +104,9 @@ mod serde_position {
         y: f32,
     }
 
-    pub fn deserialize<'de, D>(deserializer: D) -> Result<Option<Point>, D::Error>
+    pub fn deserialize<'de, D>(
+        deserializer: D,
+    ) -> Result<Option<Point>, D::Error>
     where
         D: Deserializer<'de>,
     {
@@ -126,7 +145,10 @@ mod serde_size {
         Ok(Size { width, height })
     }
 
-    pub fn serialize<S: Serializer>(size: &Size, serializer: S) -> Result<S::Ok, S::Error> {
+    pub fn serialize<S: Serializer>(
+        size: &Size,
+        serializer: S,
+    ) -> Result<S::Ok, S::Error> {
         SerdeSize {
             width: size.width,
             height: size.height,
