@@ -1,13 +1,15 @@
-use crate::widget::combo_box;
-
 pub use data::appearance::theme::{
-    color_to_hex, hex_to_color, Buffer, Button, Buttons, Colors, General, ServerMessages, Text,
+    Buffer, Button, Buttons, General, ServerMessages, Styles, Text,
+    color_to_hex, hex_to_color,
 };
 
+use crate::widget::combo_box;
+
 pub mod button;
 pub mod checkbox;
 pub mod container;
 pub mod context_menu;
+pub mod font_style;
 pub mod menu;
 pub mod pane_grid;
 pub mod progress_bar;
@@ -34,10 +36,12 @@ pub enum Theme {
 impl Theme {
     pub fn preview(&self, theme: data::Theme) -> Self {
         match self {
-            Theme::Selected(selected) | Theme::Preview { selected, .. } => Self::Preview {
-                selected: selected.clone(),
-                preview: theme,
-            },
+            Theme::Selected(selected) | Theme::Preview { selected, .. } => {
+                Self::Preview {
+                    selected: selected.clone(),
+                    preview: theme,
+                }
+            }
         }
     }
 
@@ -49,10 +53,10 @@ impl Theme {
         }
     }
 
-    pub fn colors(&self) -> &Colors {
+    pub fn styles(&self) -> &Styles {
         match self {
-            Theme::Selected(selected) => &selected.colors,
-            Theme::Preview { preview, .. } => &preview.colors,
+            Theme::Selected(selected) => &selected.styles,
+            Theme::Preview { preview, .. } => &preview.styles,
         }
     }
 }
@@ -72,10 +76,22 @@ impl Default for Theme {
 impl iced::theme::Base for Theme {
     fn base(&self) -> iced::theme::Style {
         iced::theme::Style {
-            background_color: self.colors().general.background,
-            text_color: self.colors().text.primary,
+            background_color: self.styles().general.background,
+            text_color: self.styles().text.primary.color,
         }
     }
+
+    fn palette(&self) -> Option<iced::theme::Palette> {
+        None
+    }
+    
+    fn default(_preference: iced::theme::Mode) -> Self {
+        Self::from(data::Theme::default())
+    }
+    
+    fn mode(&self) -> iced::theme::Mode {
+        iced::theme::Mode::Dark
+    }
 }
 
 impl combo_box::Catalog for Theme {}
