@@ -1,7 +1,9 @@
-use std::{io::Cursor, sync::Arc, thread};
+use std::io::Cursor;
+use std::sync::Arc;
+use std::thread;
 
 use data::audio::Sound;
-use rodio::{Decoder, OutputStream, Sink};
+use rodio::{Decoder, OutputStreamBuilder, Sink};
 
 pub fn play(sound: Sound) {
     thread::spawn(move || {
@@ -12,9 +14,8 @@ pub fn play(sound: Sound) {
 }
 
 fn _play(sound: Sound) -> Result<(), PlayError> {
-    let (_stream, stream_handle) = OutputStream::try_default()?;
-
-    let sink = Sink::try_new(&stream_handle)?;
+    let stream_handle = OutputStreamBuilder::open_default_stream()?;
+    let sink = Sink::connect_new(stream_handle.mixer());
 
     let source = Decoder::new(Cursor::new(sound))?;
 
