@@ -1,11 +1,13 @@
 use data::dashboard::BufferAction;
 use data::user::Nick;
-use data::{config, isupport, target, Config, Server, User};
-use iced::widget::{button, column, container, horizontal_rule, row, text, Space};
-use iced::{padding, Length, Padding};
+use data::{Config, Server, User, config, ctcp, isupport, target};
+use iced::widget::{
+    Space, button, column, container, horizontal_rule, row, text,
+};
+use iced::{Length, Padding, padding};
 
-use crate::widget::{context_menu, double_pass, Element};
-use crate::{theme, widget};
+use crate::widget::{Element, context_menu, double_pass};
+use crate::{Theme, font, theme, widget};
 
 #[derive(Debug, Clone, Copy)]
 pub enum Entry {
@@ -16,20 +18,28 @@ pub enum Entry {
     SendFile,
     UserInfo,
     HorizontalRule,
+    CtcpRequestTime,
+    CtcpRequestVersion,
 }
 
 impl Entry {
     pub fn list(is_channel: bool, our_user: Option<&User>) -> Vec<Self> {
         if is_channel {
-            if our_user.is_some_and(|u| u.has_access_level(data::user::AccessLevel::Oper)) {
+            if our_user.is_some_and(|u| {
+                u.has_access_level(data::user::AccessLevel::Oper)
+            }) {
                 vec![
                     Entry::UserInfo,
                     Entry::HorizontalRule,
                     Entry::Whois,
                     Entry::Query,
+                    Entry::SendFile,
+                    Entry::HorizontalRule,
                     Entry::ToggleAccessLevelOp,
                     Entry::ToggleAccessLevelVoice,
-                    Entry::SendFile,
+                    Entry::HorizontalRule,
+                    Entry::CtcpRequestVersion,
+                    Entry::CtcpRequestTime,
                 ]
             } else {
                 vec![
@@ -38,6 +48,9 @@ impl Entry {
                     Entry::Whois,
                     Entry::Query,
                     Entry::SendFile,
+                    Entry::HorizontalRule,
+                    Entry::CtcpRequestVersion,
+                    Entry::CtcpRequestTime,
                 ]
             }
         } else {
@@ -48,49 +61,63 @@ impl Entry {
     pub fn view<'a>(
         self,
         server: &Server,
-        casemapping: isupport::CaseMap,
+        prefix: &[isupport::PrefixMap],
         channel: Option<&target::Channel>,
         user: &User,
         current_user: Option<&User>,
         length: Length,
         config: &Config,
+        theme: &Theme,
     ) -> Element<'a, Message> {
         let nickname = user.nickname().to_owned();
 
         match self {
-            Entry::Whois => menu_button("Whois", Message::Whois(server.clone(), nickname), length),
+            Entry::Whois => menu_button(
+                "Whois".to_string(),
+                Message::Whois(server.clone(), nickname),
+                length,
+                theme,
+            ),
             Entry::Query => menu_button(
-                "Message",
+                "Message".to_string(),
                 Message::Query(
                     server.clone(),
-                    target::Query::from_user(user, casemapping),
+                    target::Query::from(user),
                     config.actions.buffer.message_user,
                 ),
                 length,
+                theme,
             ),
             Entry::ToggleAccessLevelOp => {
-                if let Some(channel) = channel {
+                if let (Some(channel), Some(operator_mode)) = (
+                    channel,
+                    prefix.iter().find_map(|prefix_map| {
+                        (prefix_map.prefix == '@').then_some(prefix_map.mode)
+                    }),
+                ) {
                     if user.has_access_level(data::user::AccessLevel::Oper) {
                         menu_button(
-                            "Take Op (-o)",
+                            format!("Take Op (-{operator_mode})"),
                             Message::ToggleAccessLevel(
                                 server.clone(),
                                 channel.clone(),
                                 nickname,
-                                "-o".to_owned(),
+                                format!("-{operator_mode}"),
                             ),
                             length,
+                            theme,
                         )
                     } else {
                         menu_button(
-                            "Give Op (+o)",
+                            format!("Give Op (+{operator_mode})"),
                             Message::ToggleAccessLevel(
                                 server.clone(),
                                 channel.clone(),
                                 nickname,
-                                "+o".to_owned(),
+                                format!("+{operator_mode}"),
                             ),
                             length,
+                            theme,
                         )
                     }
                 } else {
@@ -98,28 +125,35 @@ impl Entry {
                 }
             }
             Entry::ToggleAccessLevelVoice => {
-                if let Some(channel) = channel {
+                if let (Some(channel), Some(voice_mode)) = (
+                    channel,
+                    prefix.iter().find_map(|prefix_map| {
+                        (prefix_map.prefix == '+').then_some(prefix_map.mode)
+                    }),
+                ) {
                     if user.has_access_level(data::user::AccessLevel::Voice) {
                         menu_button(
-                            "Take Voice (-v)",
+                            format!("Take Voice (-{voice_mode})"),
                             Message::ToggleAccessLevel(
                                 server.clone(),
                                 channel.clone(),
                                 nickname,
-                                "-v".to_owned(),
+                                format!("-{voice_mode}"),
                             ),
                             length,
+                            theme,
                         )
                     } else {
                         menu_button(
-                            "Give Voice (+v)",
+                            format!("Give Voice (+{voice_mode})"),
                             Message::ToggleAccessLevel(
                                 server.clone(),
                                 channel.clone(),
                                 nickname,
-                                "+v".to_owned(),
+                                format!("+{voice_mode}"),
                             ),
                             length,
+                            theme,
                         )
                     }
                 } else {
@@ -127,15 +161,42 @@ impl Entry {
                 }
             }
             Entry::SendFile => menu_button(
-                "Send File",
-                Message::SendFile(server.clone(), nickname),
+                "Send File".to_string(),
+                Message::SendFile(server.clone(), user.clone()),
                 length,
+                theme,
             ),
-            Entry::UserInfo => user_info(current_user, nickname, length, config),
+            Entry::UserInfo => {
+                user_info(current_user, nickname, length, config, theme)
+            }
             Entry::HorizontalRule => match length {
-                Length::Fill => container(horizontal_rule(1)).padding([0, 6]).into(),
+                Length::Fill => {
+                    container(horizontal_rule(1)).padding([0, 6]).into()
+                }
                 _ => Space::new(length, 1).into(),
             },
+            Entry::CtcpRequestTime => menu_button(
+                "Local Time (TIME)".to_string(),
+                Message::CtcpRequest(
+                    ctcp::Command::Time,
+                    server.clone(),
+                    nickname,
+                    None,
+                ),
+                length,
+                theme,
+            ),
+            Entry::CtcpRequestVersion => menu_button(
+                "Client (VERSION)".to_string(),
+                Message::CtcpRequest(
+                    ctcp::Command::Version,
+                    server.clone(),
+                    nickname,
+                    None,
+                ),
+                length,
+                theme,
+            ),
         }
     }
 }
@@ -145,8 +206,9 @@ pub enum Message {
     Whois(Server, Nick),
     Query(Server, target::Query, BufferAction),
     ToggleAccessLevel(Server, target::Channel, Nick, String),
-    SendFile(Server, Nick),
+    SendFile(Server, User),
     InsertNickname(Nick),
+    CtcpRequest(ctcp::Command, Server, Nick, Option<String>),
 }
 
 #[derive(Debug, Clone)]
@@ -154,33 +216,38 @@ pub enum Event {
     SendWhois(Server, Nick),
     OpenQuery(Server, target::Query, BufferAction),
     ToggleAccessLevel(Server, target::Channel, Nick, String),
-    SendFile(Server, Nick),
+    SendFile(Server, User),
     InsertNickname(Nick),
+    CtcpRequest(ctcp::Command, Server, Nick, Option<String>),
 }
 
-pub fn update(message: Message) -> Option<Event> {
+pub fn update(message: Message) -> Event {
     match message {
-        Message::Whois(server, nick) => Some(Event::SendWhois(server, nick)),
+        Message::Whois(server, nick) => Event::SendWhois(server, nick),
         Message::Query(server, nick, buffer_action) => {
-            Some(Event::OpenQuery(server, nick, buffer_action))
+            Event::OpenQuery(server, nick, buffer_action)
         }
         Message::ToggleAccessLevel(server, target, nick, mode) => {
-            Some(Event::ToggleAccessLevel(server, target, nick, mode))
+            Event::ToggleAccessLevel(server, target, nick, mode)
+        }
+        Message::SendFile(server, user) => Event::SendFile(server, user),
+        Message::InsertNickname(nick) => Event::InsertNickname(nick),
+        Message::CtcpRequest(command, server, nick, params) => {
+            Event::CtcpRequest(command, server, nick, params)
         }
-        Message::SendFile(server, nick) => Some(Event::SendFile(server, nick)),
-        Message::InsertNickname(nick) => Some(Event::InsertNickname(nick)),
     }
 }
 
 pub fn view<'a>(
     content: impl Into<Element<'a, Message>>,
     server: &'a Server,
-    casemapping: isupport::CaseMap,
+    prefix: &'a [isupport::PrefixMap],
     channel: Option<&'a target::Channel>,
     user: &'a User,
     current_user: Option<&'a User>,
     our_user: Option<&'a User>,
     config: &'a Config,
+    theme: &'a Theme,
     click: &'a config::buffer::NicknameClickAction,
 ) -> Element<'a, Message> {
     let entries = Entry::list(channel.is_some(), our_user);
@@ -188,7 +255,7 @@ pub fn view<'a>(
     let message = match click {
         data::config::buffer::NicknameClickAction::OpenQuery => Message::Query(
             server.clone(),
-            target::Query::from_user(user, casemapping),
+            target::Query::from(user),
             config.actions.buffer.click_username,
         ),
         data::config::buffer::NicknameClickAction::InsertNickname => {
@@ -200,29 +267,41 @@ pub fn view<'a>(
 
     context_menu(
         context_menu::MouseButton::default(),
+        context_menu::Anchor::Cursor,
+        context_menu::ToggleBehavior::KeepOpen,
         base,
         entries,
         move |entry, length| {
             entry.view(
                 server,
-                casemapping,
+                prefix,
                 channel,
                 user,
                 current_user,
                 length,
                 config,
+                theme,
             )
         },
     )
     .into()
 }
 
-fn menu_button(content: &str, message: Message, length: Length) -> Element<'_, Message> {
-    button(text(content).style(theme::text::primary))
-        .padding(5)
-        .width(length)
-        .on_press(message)
-        .into()
+fn menu_button(
+    content: String,
+    message: Message,
+    length: Length,
+    theme: &Theme,
+) -> Element<'static, Message> {
+    button(
+        text(content)
+            .style(theme::text::primary)
+            .font_maybe(theme::font_style::primary(theme).map(font::get)),
+    )
+    .padding(5)
+    .width(length)
+    .on_press(message)
+    .into()
 }
 
 fn right_justified_padding() -> Padding {
@@ -234,32 +313,61 @@ fn user_info<'a>(
     nickname: Nick,
     length: Length,
     config: &Config,
+    theme: &Theme,
 ) -> Element<'a, Message> {
     let state = match current_user {
         Some(user) => {
             if user.is_away() {
-                Some(text("Away").style(theme::text::secondary).width(length))
+                Some(
+                    text("(Away)")
+                        .style(theme::text::secondary)
+                        .font_maybe(
+                            theme::font_style::secondary(theme).map(font::get),
+                        )
+                        .width(length),
+                )
             } else {
                 None
             }
         }
-        None => Some(text("Offline").style(theme::text::secondary).width(length)),
+        None => Some(
+            text("(Offline)")
+                .style(theme::text::secondary)
+                .font_maybe(theme::font_style::secondary(theme).map(font::get))
+                .width(length),
+        ),
     };
 
     // Dimmed if away or offline.
-    let is_user_away = current_user.is_some_and(|u| u.is_away());
-    let away_appearance = config.buffer.away.appearance(is_user_away);
+    let is_user_away = config
+        .buffer
+        .nickname
+        .away
+        .is_away(current_user.is_none_or(User::is_away));
+    let is_user_offline = config
+        .buffer
+        .nickname
+        .offline
+        .is_offline(current_user.is_none());
     let seed = match config.buffer.nickname.color {
         data::buffer::Color::Solid => None,
         data::buffer::Color::Unique => Some(nickname.to_string()),
     };
 
-    column![container(
-        text(nickname.to_string())
-            .style(move |theme| theme::text::nickname(theme, seed.clone(), away_appearance))
-            .width(length)
-    )
-    .padding(right_justified_padding()),]
-    .push_maybe(state.map(|s| container(s).padding(right_justified_padding())))
+    let style = theme::text::nickname(
+        theme,
+        seed.clone(),
+        is_user_away,
+        is_user_offline,
+    );
+
+    let nickname = text(nickname.to_string()).style(move |_| style).font_maybe(
+        theme::font_style::nickname(theme, is_user_offline).map(font::get),
+    );
+
+    column![
+        container(row![nickname, state].width(length).spacing(4))
+            .padding(right_justified_padding())
+    ]
     .into()
 }
