@@ -1,5 +1,7 @@
-use iced::advanced::{layout, overlay, renderer, widget, Clipboard, Layout, Shell, Widget};
-use iced::{mouse, Event, Length, Point, Rectangle, Size, Vector};
+use iced::advanced::{
+    Clipboard, Layout, Shell, Widget, layout, overlay, renderer, widget,
+};
+use iced::{Event, Length, Point, Rectangle, Size, Vector, mouse};
 
 use super::{Element, Renderer};
 use crate::Theme;
@@ -32,7 +34,9 @@ struct AnchoredOverlay<'a, Message> {
     offset: f32,
 }
 
-impl<Message> Widget<Message, Theme, Renderer> for AnchoredOverlay<'_, Message> {
+impl<Message> Widget<Message, Theme, Renderer>
+    for AnchoredOverlay<'_, Message>
+{
     fn size(&self) -> Size<Length> {
         self.base.as_widget().size()
     }
@@ -42,13 +46,13 @@ impl<Message> Widget<Message, Theme, Renderer> for AnchoredOverlay<'_, Message>
     }
 
     fn layout(
-        &self,
+        &mut self,
         tree: &mut widget::Tree,
         renderer: &Renderer,
         limits: &layout::Limits,
     ) -> layout::Node {
         self.base
-            .as_widget()
+            .as_widget_mut()
             .layout(&mut tree.children[0], renderer, limits)
     }
 
@@ -85,15 +89,18 @@ impl<Message> Widget<Message, Theme, Renderer> for AnchoredOverlay<'_, Message>
     }
 
     fn operate(
-        &self,
+        &mut self,
         tree: &mut iced::advanced::widget::Tree,
         layout: Layout<'_>,
         renderer: &Renderer,
         operation: &mut dyn widget::Operation<()>,
     ) {
-        self.base
-            .as_widget()
-            .operate(&mut tree.children[0], layout, renderer, operation);
+        self.base.as_widget_mut().operate(
+            &mut tree.children[0],
+            layout,
+            renderer,
+            operation,
+        );
     }
 
     fn update(
@@ -139,16 +146,20 @@ impl<Message> Widget<Message, Theme, Renderer> for AnchoredOverlay<'_, Message>
     fn overlay<'b>(
         &'b mut self,
         tree: &'b mut widget::Tree,
-        layout: Layout<'_>,
+        layout: Layout<'b>,
         renderer: &Renderer,
+        viewport: &Rectangle,
         translation: Vector,
     ) -> Option<overlay::Element<'b, Message, Theme, Renderer>> {
         let (first, second) = tree.children.split_at_mut(1);
 
-        let base = self
-            .base
-            .as_widget_mut()
-            .overlay(&mut first[0], layout, renderer, translation);
+        let base = self.base.as_widget_mut().overlay(
+            &mut first[0],
+            layout,
+            renderer,
+            viewport,
+            translation,
+        );
 
         let overlay = overlay::Element::new(Box::new(Overlay {
             content: &mut self.overlay,
@@ -157,11 +168,14 @@ impl<Message> Widget<Message, Theme, Renderer> for AnchoredOverlay<'_, Message>
             offset: self.offset,
             base_layout: layout.bounds(),
             position: layout.position(),
+            viewport: *viewport,
         }));
 
         Some(
-            overlay::Group::with_children(base.into_iter().chain(Some(overlay)).collect())
-                .overlay(),
+            overlay::Group::with_children(
+                base.into_iter().chain(Some(overlay)).collect(),
+            )
+            .overlay(),
         )
     }
 }
@@ -182,9 +196,12 @@ struct Overlay<'a, 'b, Message> {
     offset: f32,
     base_layout: Rectangle,
     position: Point,
+    viewport: Rectangle,
 }
 
-impl<Message> overlay::Overlay<Message, Theme, Renderer> for Overlay<'_, '_, Message> {
+impl<Message> overlay::Overlay<Message, Theme, Renderer>
+    for Overlay<'_, '_, Message>
+{
     fn layout(&mut self, renderer: &Renderer, bounds: Size) -> layout::Node {
         let height = match self.anchor {
             // From top of base to top of viewport
@@ -205,12 +222,14 @@ impl<Message> overlay::Overlay<Message, Theme, Renderer> for Overlay<'_, '_, Mes
 
         let node = self
             .content
-            .as_widget()
+            .as_widget_mut()
             .layout(self.tree, renderer, &limits);
 
         let translation = match self.anchor {
             // Overlay height + offset above the top
-            Anchor::AboveTop => Vector::new(0.0, -(node.size().height + self.offset)),
+            Anchor::AboveTop => {
+                Vector::new(0.0, -(node.size().height + self.offset))
+            }
             // Offset below the top and centered
             Anchor::BelowTopCentered => Vector::new(
                 self.base_layout.width / 2.0 - node.size().width / 2.0,
@@ -276,25 +295,28 @@ impl<Message> overlay::Overlay<Message, Theme, Renderer> for Overlay<'_, '_, Mes
         &self,
         layout: Layout<'_>,
         cursor: mouse::Cursor,
-        viewport: &Rectangle,
         renderer: &Renderer,
     ) -> iced::advanced::mouse::Interaction {
-        self.content
-            .as_widget()
-            .mouse_interaction(self.tree, layout, cursor, viewport, renderer)
-    }
-
-    fn is_over(&self, layout: Layout<'_>, _renderer: &Renderer, cursor_position: Point) -> bool {
-        layout.bounds().contains(cursor_position)
+        self.content.as_widget().mouse_interaction(
+            self.tree,
+            layout,
+            cursor,
+            &layout.bounds(),
+            renderer,
+        )
     }
 
     fn overlay<'c>(
         &'c mut self,
-        layout: Layout<'_>,
+        layout: Layout<'c>,
         renderer: &Renderer,
     ) -> Option<overlay::Element<'c, Message, Theme, Renderer>> {
-        self.content
-            .as_widget_mut()
-            .overlay(self.tree, layout, renderer, Vector::default())
+        self.content.as_widget_mut().overlay(
+            self.tree,
+            layout,
+            renderer,
+            &self.viewport,
+            Vector::default(),
+        )
     }
 }
