@@ -1,11 +1,14 @@
 use std::slice;
 
-use iced::advanced::widget::{operation, tree, Operation};
-use iced::advanced::{self, layout, overlay, renderer, widget, Clipboard, Layout, Shell, Widget};
-use iced::widget::{column, container};
-use iced::{mouse, Element, Event, Length, Point, Rectangle, Size, Task, Vector};
-
+use iced::advanced::widget::{Operation, operation, tree};
+use iced::advanced::{
+    self, Clipboard, Layout, Shell, Widget, layout, overlay, renderer, widget,
+};
 pub use iced::widget::container::{Style, StyleFn};
+use iced::widget::{column, container};
+use iced::{
+    Element, Event, Length, Point, Rectangle, Size, Task, Vector, mouse,
+};
 
 use super::double_pass;
 
@@ -16,8 +19,24 @@ pub enum MouseButton {
     Right,
 }
 
+#[derive(Debug, Default, Clone, Copy)]
+pub enum Anchor {
+    #[default]
+    Cursor,
+    Widget,
+}
+
+#[derive(Debug, Default, Clone, Copy)]
+pub enum ToggleBehavior {
+    #[default]
+    KeepOpen,
+    Close,
+}
+
 pub fn context_menu<'a, T, Message, Theme, Renderer>(
     activation_button: MouseButton,
+    anchor: Anchor,
+    toggle_behavior: ToggleBehavior,
     base: impl Into<Element<'a, Message, Theme, Renderer>>,
     entries: Vec<T>,
     entry: impl Fn(T, Length) -> Element<'a, Message, Theme, Renderer> + 'a,
@@ -30,7 +49,8 @@ pub fn context_menu<'a, T, Message, Theme, Renderer>(
             MouseButton::Left => iced::mouse::Button::Left,
             MouseButton::Right => iced::mouse::Button::Right,
         },
-
+        anchor,
+        toggle_behavior,
         menu: None,
     }
 }
@@ -40,8 +60,9 @@ pub struct ContextMenu<'a, T, Message, Theme, Renderer> {
     entries: Vec<T>,
     entry: Box<dyn Fn(T, Length) -> Element<'a, Message, Theme, Renderer> + 'a>,
     activation_button: iced::mouse::Button,
-
-    // Cached, recreated during `overlay` if menu is open
+    anchor: Anchor,
+    toggle_behavior: ToggleBehavior,
+    // Cached, recreated during overlay if menu is open
     menu: Option<Element<'a, Message, Theme, Renderer>>,
 }
 
@@ -54,7 +75,7 @@ pub struct State {
 impl State {
     pub fn new() -> Self {
         State {
-            status: Status::Closed,
+            status: Status::Closed(false),
             menu_tree: widget::Tree::empty(),
         }
     }
@@ -62,14 +83,14 @@ impl State {
 
 #[derive(Debug, Clone, Copy, PartialEq)]
 pub enum Status {
-    Closed,
+    Closed(bool),
     Open(Point),
 }
 
 impl Status {
     pub fn open(self) -> Option<Point> {
         match self {
-            Status::Closed => None,
+            Status::Closed(_) => None,
             Status::Open(position) => Some(position),
         }
     }
@@ -81,7 +102,8 @@ where
     T: Copy + 'a,
     Message: 'a,
     Theme: 'a + container::Catalog + Catalog,
-    <Theme as container::Catalog>::Class<'a>: From<container::StyleFn<'a, Theme>>,
+    <Theme as container::Catalog>::Class<'a>:
+        From<container::StyleFn<'a, Theme>>,
     Renderer: advanced::Renderer + 'a,
 {
     fn size(&self) -> Size<Length> {
@@ -93,13 +115,13 @@ where
     }
 
     fn layout(
-        &self,
+        &mut self,
         tree: &mut widget::Tree,
         renderer: &Renderer,
         limits: &layout::Limits,
     ) -> layout::Node {
         self.base
-            .as_widget()
+            .as_widget_mut()
             .layout(&mut tree.children[0], renderer, limits)
     }
 
@@ -130,7 +152,7 @@ where
 
     fn state(&self) -> tree::State {
         tree::State::new(State {
-            status: Status::Closed,
+            status: Status::Closed(false),
             menu_tree: widget::Tree::empty(),
         })
     }
@@ -144,7 +166,7 @@ where
     }
 
     fn operate(
-        &self,
+        &mut self,
         tree: &mut iced::advanced::widget::Tree,
         layout: Layout<'_>,
         renderer: &Renderer,
@@ -154,9 +176,12 @@ where
 
         operation.custom(None, layout.bounds(), state);
 
-        self.base
-            .as_widget()
-            .operate(&mut tree.children[0], layout, renderer, operation);
+        self.base.as_widget_mut().operate(
+            &mut tree.children[0],
+            layout,
+            renderer,
+            operation,
+        );
     }
 
     fn update(
@@ -170,36 +195,76 @@ where
         shell: &mut Shell<'_, Message>,
         viewport: &Rectangle,
     ) {
-        let state = tree.state.downcast_mut::<State>();
-        let prev_status = state.status;
+        // is this a mouse event we are waiting for?
+        let is_mouse_event = matches!(
+            event,
+            Event::Mouse(mouse::Event::ButtonPressed(mouse::Button::Left))
+                | Event::Mouse(mouse::Event::ButtonPressed(
+                    mouse::Button::Right
+                ))
+        );
 
-        let position = match self.activation_button {
-            mouse::Button::Left => {
-                if let Event::Mouse(mouse::Event::ButtonReleased(mouse::Button::Left)) = event {
-                    cursor.position_over(layout.bounds())
-                } else {
-                    None
+        if is_mouse_event {
+            let state = tree.state.downcast_mut::<State>();
+            let prev_status = state.status;
+
+            // is this a mouse event for that we should do something?
+            let is_activation_mouse_event = *event
+                == Event::Mouse(mouse::Event::ButtonPressed(
+                    self.activation_button,
+                ));
+
+            let position = if is_activation_mouse_event {
+                match self.anchor {
+                    Anchor::Widget => {
+                        cursor.position_over(layout.bounds()).map(|_| {
+                            let widget = layout.bounds();
+                            Point::new(
+                                widget.x + widget.width,
+                                widget.y + widget.height,
+                            )
+                        })
+                    }
+                    Anchor::Cursor => {
+                        cursor.position_over(layout.bounds()).map(|cursor| {
+                            Point::new(cursor.x + 5.0, cursor.y + 5.0)
+                        })
+                    }
+                }
+            } else {
+                None
+            };
+
+            // determinate next status
+            let next_status = match (
+                is_activation_mouse_event,
+                prev_status,
+                self.toggle_behavior,
+                position,
+            ) {
+                (true, _, ToggleBehavior::KeepOpen, Some(position))
+                | (
+                    true,
+                    Status::Closed(false),
+                    ToggleBehavior::Close,
+                    Some(position),
+                ) => Status::Open(position),
+
+                (false, Status::Open(_), _, None)
+                | (true, Status::Open(_), _, None)
+                | (true, Status::Open(_), ToggleBehavior::Close, Some(_))
+                | (_, Status::Closed(true), _, _) => Status::Closed(false),
+                _ => prev_status, // keep status
+            };
+
+            if next_status != prev_status {
+                state.status = next_status;
+                if !matches!(next_status, Status::Closed(_))
+                    || !matches!(prev_status, Status::Closed(_))
+                {
+                    shell.request_redraw();
                 }
             }
-            mouse::Button::Right => {
-                if let Event::Mouse(mouse::Event::ButtonPressed(mouse::Button::Right)) = event {
-                    cursor.position_over(layout.bounds())
-                } else {
-                    None
-                }
-            }
-            _ => None,
-        };
-
-        if let Some(position) = position {
-            state.status = Status::Open(position);
-        }
-
-        match (state.status, prev_status) {
-            (Status::Closed, Status::Open(_)) | (Status::Open(_), Status::Closed) => {
-                shell.request_redraw();
-            }
-            _ => {}
         }
 
         self.base.as_widget_mut().update(
@@ -222,24 +287,29 @@ where
         _viewport: &Rectangle,
         _renderer: &Renderer,
     ) -> mouse::Interaction {
-        cursor
-            .is_over(layout.bounds())
-            .then_some(mouse::Interaction::Pointer)
-            .unwrap_or_default()
+        if cursor.is_over(layout.bounds()) {
+            mouse::Interaction::Pointer
+        } else {
+            mouse::Interaction::default()
+        }
     }
 
     fn overlay<'b>(
         &'b mut self,
         tree: &'b mut widget::Tree,
-        layout: Layout<'_>,
+        layout: Layout<'b>,
         renderer: &Renderer,
+        viewport: &Rectangle,
         translation: Vector,
     ) -> Option<overlay::Element<'b, Message, Theme, Renderer>> {
         let base_state = tree.children.first_mut().unwrap();
-        let base = self
-            .base
-            .as_widget_mut()
-            .overlay(base_state, layout, renderer, translation);
+        let base = self.base.as_widget_mut().overlay(
+            base_state,
+            layout,
+            renderer,
+            viewport,
+            translation,
+        );
 
         let state = tree.state.downcast_mut::<State>();
 
@@ -254,7 +324,12 @@ where
         if base.is_none() && overlay.is_none() {
             None
         } else {
-            Some(overlay::Group::with_children(base.into_iter().chain(overlay).collect()).overlay())
+            Some(
+                overlay::Group::with_children(
+                    base.into_iter().chain(overlay).collect(),
+                )
+                .overlay(),
+            )
         }
     }
 }
@@ -267,16 +342,22 @@ where
     T: Copy + 'a,
     Message: 'a,
     Theme: 'a + container::Catalog + Catalog,
-    <Theme as container::Catalog>::Class<'a>: From<container::StyleFn<'a, Theme>>,
+    <Theme as container::Catalog>::Class<'a>:
+        From<container::StyleFn<'a, Theme>>,
     Renderer: advanced::Renderer + 'a,
 {
     let build_menu =
-        |length, view: &(dyn Fn(T, Length) -> Element<'a, Message, Theme, Renderer> + 'a)| {
+        |length,
+         view: &(
+              dyn Fn(T, Length) -> Element<'a, Message, Theme, Renderer> + 'a
+          )| {
             container(column(
                 entries.iter().copied().map(|entry| view(entry, length)),
             ))
             .padding(4)
-            .style(|theme| <Theme as Catalog>::style(theme, &<Theme as Catalog>::default()))
+            .style(|theme| {
+                <Theme as Catalog>::style(theme, &<Theme as Catalog>::default())
+            })
         };
 
     double_pass(
@@ -296,7 +377,8 @@ where
     T: Copy + 'a,
     Message: 'a,
     Theme: 'a + container::Catalog + Catalog,
-    <Theme as container::Catalog>::Class<'a>: From<container::StyleFn<'a, Theme>>,
+    <Theme as container::Catalog>::Class<'a>:
+        From<container::StyleFn<'a, Theme>>,
     Renderer: advanced::Renderer + 'a,
 {
     if entries.is_empty() {
@@ -313,7 +395,7 @@ where
                 *menu = Some(_menu);
             }
         },
-        Status::Closed => {
+        Status::Closed(_) => {
             *menu = None;
         }
     }
@@ -353,11 +435,11 @@ pub fn close<Message: 'static + Send>(f: fn(bool) -> Message) -> Task<Message> {
             _bounds: Rectangle,
             state: &mut dyn std::any::Any,
         ) {
-            if let Some(state) = state.downcast_mut::<State>() {
-                if let Status::Open(_) = state.status {
-                    state.status = Status::Closed;
-                    self.any_closed = true;
-                }
+            if let Some(state) = state.downcast_mut::<State>()
+                && let Status::Open(_) = state.status
+            {
+                state.status = Status::Closed(true);
+                self.any_closed = true;
             }
         }
 
@@ -372,16 +454,20 @@ pub fn close<Message: 'static + Send>(f: fn(bool) -> Message) -> Task<Message> {
     })
 }
 
-impl<'a, T, Message, Theme, Renderer> From<ContextMenu<'a, T, Message, Theme, Renderer>>
+impl<'a, T, Message, Theme, Renderer>
+    From<ContextMenu<'a, T, Message, Theme, Renderer>>
     for Element<'a, Message, Theme, Renderer>
 where
     T: Copy + 'a,
     Message: 'a,
     Theme: 'a + container::Catalog + Catalog,
-    <Theme as container::Catalog>::Class<'a>: From<container::StyleFn<'a, Theme>>,
+    <Theme as container::Catalog>::Class<'a>:
+        From<container::StyleFn<'a, Theme>>,
     Renderer: advanced::Renderer + 'a,
 {
-    fn from(context_menu: ContextMenu<'a, T, Message, Theme, Renderer>) -> Self {
+    fn from(
+        context_menu: ContextMenu<'a, T, Message, Theme, Renderer>,
+    ) -> Self {
         Element::new(context_menu)
     }
 }
@@ -402,16 +488,20 @@ where
             .width(Length::Fill)
             .height(Length::Fill);
 
-        let node = self
-            .menu
-            .as_widget()
-            .layout(&mut self.state.menu_tree, renderer, &limits);
+        let node = self.menu.as_widget_mut().layout(
+            &mut self.state.menu_tree,
+            renderer,
+            &limits,
+        );
 
         // Small padding to ensure that we don't spawn context menu at the very edge of the viewport.
         let padding = 5.0;
         let viewport = Rectangle::new(
             Point::new(Point::ORIGIN.x + padding, Point::ORIGIN.y + padding),
-            Size::new(bounds.width - 2.0 * padding, bounds.height - 2.0 * padding),
+            Size::new(
+                bounds.width - 2.0 * padding,
+                bounds.height - 2.0 * padding,
+            ),
         );
         let mut bounds = Rectangle::new(self.position, node.size());
 
@@ -455,9 +545,12 @@ where
         renderer: &Renderer,
         operation: &mut dyn widget::Operation<()>,
     ) {
-        self.menu
-            .as_widget_mut()
-            .operate(&mut self.state.menu_tree, layout, renderer, operation);
+        self.menu.as_widget_mut().operate(
+            &mut self.state.menu_tree,
+            layout,
+            renderer,
+            operation,
+        );
     }
 
     fn update(
@@ -469,20 +562,17 @@ where
         clipboard: &mut dyn Clipboard,
         shell: &mut Shell<'_, Message>,
     ) {
-        if let Event::Mouse(mouse::Event::ButtonPressed(_)) = &event {
-            if cursor.position_over(layout.bounds()).is_none() {
-                self.state.status = Status::Closed;
-            }
+        if let Event::Mouse(mouse::Event::ButtonPressed(_)) = &event
+            && cursor.position_over(layout.bounds()).is_none()
+        {
+            self.state.status = Status::Closed(true);
         }
 
-        if let Event::Mouse(mouse::Event::ButtonReleased(mouse::Button::Left)) = &event {
-            if cursor.position_over(layout.bounds()).is_some() {
-                self.state.status = Status::Closed;
-            }
-        }
-
-        if matches!(self.state.status, Status::Closed) {
-            shell.request_redraw();
+        if let Event::Mouse(mouse::Event::ButtonReleased(mouse::Button::Left)) =
+            &event
+            && cursor.position_over(layout.bounds()).is_some()
+        {
+            self.state.status = Status::Closed(true);
         }
 
         self.menu.as_widget_mut().update(
@@ -501,21 +591,16 @@ where
         &self,
         layout: Layout<'_>,
         cursor: mouse::Cursor,
-        viewport: &Rectangle,
         renderer: &Renderer,
     ) -> iced::advanced::mouse::Interaction {
         self.menu.as_widget().mouse_interaction(
             &self.state.menu_tree,
             layout,
             cursor,
-            viewport,
+            &layout.bounds(),
             renderer,
         )
     }
-
-    fn is_over(&self, layout: Layout<'_>, _renderer: &Renderer, cursor_position: Point) -> bool {
-        layout.bounds().contains(cursor_position)
-    }
 }
 
 /// The theme catalog of a [`Catalog`].
