@@ -1,9 +1,8 @@
-use std::{marker::PhantomData, slice};
+use std::marker::PhantomData;
+use std::slice;
 
-use iced::{
-    advanced::{self, layout, Widget},
-    Element,
-};
+use iced::advanced::{self, Widget, layout};
+use iced::{Element, Rectangle};
 
 pub fn decorate<'a, Message, Theme, Renderer>(
     element: impl Into<Element<'a, Message, Theme, Renderer>>,
@@ -35,7 +34,9 @@ pub struct Decorate<
 }
 
 impl<'a, Message, Theme, Renderer> Decorate<'a, Message, Theme, Renderer> {
-    pub fn new(inner: impl Into<Element<'a, Message, Theme, Renderer>>) -> Self {
+    pub fn new(
+        inner: impl Into<Element<'a, Message, Theme, Renderer>>,
+    ) -> Self {
         Self {
             inner: inner.into(),
             update: (),
@@ -50,18 +51,18 @@ impl<'a, Message, Theme, Renderer> Decorate<'a, Message, Theme, Renderer> {
 }
 
 impl<
-        'a,
-        Message,
-        Theme,
-        Renderer,
-        Layout,
-        Update,
-        Draw,
-        MouseInteraction,
-        Operate,
-        Overlay,
-        State,
-    >
+    'a,
+    Message,
+    Theme,
+    Renderer,
+    Layout,
+    Update,
+    Draw,
+    MouseInteraction,
+    Operate,
+    Overlay,
+    State,
+>
     Decorate<
         'a,
         Message,
@@ -172,7 +173,19 @@ impl<
     pub fn mouse_interaction<T, U>(
         self,
         mouse_interaction: T,
-    ) -> Decorate<'a, Message, Theme, Renderer, Layout, Update, Draw, T, Operate, Overlay, U>
+    ) -> Decorate<
+        'a,
+        Message,
+        Theme,
+        Renderer,
+        Layout,
+        Update,
+        Draw,
+        T,
+        Operate,
+        Overlay,
+        U,
+    >
     where
         T: self::MouseInteraction<'a, Message, Theme, Renderer, U>,
     {
@@ -191,7 +204,19 @@ impl<
     pub fn operate<T, U>(
         self,
         operate: T,
-    ) -> Decorate<'a, Message, Theme, Renderer, Layout, Update, Draw, MouseInteraction, T, Overlay, U>
+    ) -> Decorate<
+        'a,
+        Message,
+        Theme,
+        Renderer,
+        Layout,
+        Update,
+        Draw,
+        MouseInteraction,
+        T,
+        Overlay,
+        U,
+    >
     where
         T: self::Operate<'a, Message, Theme, Renderer, U>,
     {
@@ -210,7 +235,19 @@ impl<
     pub fn overlay<T, U>(
         self,
         overlay: T,
-    ) -> Decorate<'a, Message, Theme, Renderer, Layout, Update, Draw, MouseInteraction, Operate, T, U>
+    ) -> Decorate<
+        'a,
+        Message,
+        Theme,
+        Renderer,
+        Layout,
+        Update,
+        Draw,
+        MouseInteraction,
+        Operate,
+        T,
+        U,
+    >
     where
         T: self::Operate<'a, Message, Theme, Renderer, U>,
     {
@@ -229,36 +266,38 @@ impl<
 
 pub trait Layout<'a, Message, Theme, Renderer, State> {
     fn layout(
-        &self,
+        &mut self,
         state: &mut State,
-        inner: &Element<'a, Message, Theme, Renderer>,
+        inner: &mut Element<'a, Message, Theme, Renderer>,
         tree: &mut iced::advanced::widget::Tree,
         renderer: &Renderer,
         limits: &iced::advanced::layout::Limits,
     ) -> layout::Node;
 }
 
-impl<'a, Message, Theme, Renderer, State> Layout<'a, Message, Theme, Renderer, State> for ()
+impl<'a, Message, Theme, Renderer, State>
+    Layout<'a, Message, Theme, Renderer, State> for ()
 where
     Renderer: advanced::Renderer + 'a,
 {
     fn layout(
-        &self,
+        &mut self,
         _state: &mut State,
-        inner: &Element<'a, Message, Theme, Renderer>,
+        inner: &mut Element<'a, Message, Theme, Renderer>,
         tree: &mut iced::advanced::widget::Tree,
         renderer: &Renderer,
         limits: &iced::advanced::layout::Limits,
     ) -> layout::Node {
-        inner.as_widget().layout(tree, renderer, limits)
+        inner.as_widget_mut().layout(tree, renderer, limits)
     }
 }
 
-impl<'a, T, Message, Theme, Renderer, State> Layout<'a, Message, Theme, Renderer, State> for T
+impl<'a, T, Message, Theme, Renderer, State>
+    Layout<'a, Message, Theme, Renderer, State> for T
 where
     T: Fn(
             &mut State,
-            &Element<'a, Message, Theme, Renderer>,
+            &mut Element<'a, Message, Theme, Renderer>,
             &mut iced::advanced::widget::Tree,
             &Renderer,
             &iced::advanced::layout::Limits,
@@ -266,9 +305,9 @@ where
         + 'a,
 {
     fn layout(
-        &self,
+        &mut self,
         state: &mut State,
-        inner: &Element<'a, Message, Theme, Renderer>,
+        inner: &mut Element<'a, Message, Theme, Renderer>,
         tree: &mut iced::advanced::widget::Tree,
         renderer: &Renderer,
         limits: &iced::advanced::layout::Limits,
@@ -293,7 +332,8 @@ pub trait Update<'a, Message, Theme, Renderer, State> {
     );
 }
 
-impl<'a, Message, Theme, Renderer, State> Update<'a, Message, Theme, Renderer, State> for ()
+impl<'a, Message, Theme, Renderer, State>
+    Update<'a, Message, Theme, Renderer, State> for ()
 where
     Renderer: advanced::Renderer + 'a,
 {
@@ -316,7 +356,8 @@ where
     }
 }
 
-impl<'a, T, Message, Theme, Renderer, State> Update<'a, Message, Theme, Renderer, State> for T
+impl<'a, T, Message, Theme, Renderer, State>
+    Update<'a, Message, Theme, Renderer, State> for T
 where
     T: Fn(
             &mut State,
@@ -345,7 +386,8 @@ where
         viewport: &iced::Rectangle,
     ) {
         self(
-            state, inner, tree, event, layout, cursor, renderer, clipboard, shell, viewport,
+            state, inner, tree, event, layout, cursor, renderer, clipboard,
+            shell, viewport,
         );
     }
 }
@@ -365,7 +407,8 @@ pub trait Draw<'a, Message, Theme, Renderer, State> {
     );
 }
 
-impl<'a, Message, Theme, Renderer, State> Draw<'a, Message, Theme, Renderer, State> for ()
+impl<'a, Message, Theme, Renderer, State>
+    Draw<'a, Message, Theme, Renderer, State> for ()
 where
     Renderer: advanced::Renderer + 'a,
 {
@@ -387,7 +430,8 @@ where
     }
 }
 
-impl<'a, T, Message, Theme, Renderer, State> Draw<'a, Message, Theme, Renderer, State> for T
+impl<'a, T, Message, Theme, Renderer, State>
+    Draw<'a, Message, Theme, Renderer, State> for T
 where
     T: Fn(
             &State,
@@ -414,7 +458,8 @@ where
         viewport: &iced::Rectangle,
     ) {
         self(
-            state, inner, tree, renderer, theme, style, layout, cursor, viewport,
+            state, inner, tree, renderer, theme, style, layout, cursor,
+            viewport,
         );
     }
 }
@@ -432,8 +477,8 @@ pub trait MouseInteraction<'a, Message, Theme, Renderer, State> {
     ) -> advanced::mouse::Interaction;
 }
 
-impl<'a, Message, Theme, Renderer, State> MouseInteraction<'a, Message, Theme, Renderer, State>
-    for ()
+impl<'a, Message, Theme, Renderer, State>
+    MouseInteraction<'a, Message, Theme, Renderer, State> for ()
 where
     Renderer: advanced::Renderer + 'a,
 {
@@ -453,8 +498,8 @@ where
     }
 }
 
-impl<'a, T, Message, Theme, Renderer, State> MouseInteraction<'a, Message, Theme, Renderer, State>
-    for T
+impl<'a, T, Message, Theme, Renderer, State>
+    MouseInteraction<'a, Message, Theme, Renderer, State> for T
 where
     T: Fn(
             &State,
@@ -485,7 +530,7 @@ pub trait Operate<'a, Message, Theme, Renderer, State> {
     fn operate(
         &self,
         state: &mut State,
-        inner: &Element<'a, Message, Theme, Renderer>,
+        inner: &mut Element<'a, Message, Theme, Renderer>,
         tree: &mut advanced::widget::Tree,
         layout: advanced::Layout<'_>,
         renderer: &Renderer,
@@ -493,28 +538,30 @@ pub trait Operate<'a, Message, Theme, Renderer, State> {
     );
 }
 
-impl<'a, Message, Theme, Renderer, State> Operate<'a, Message, Theme, Renderer, State> for ()
+impl<'a, Message, Theme, Renderer, State>
+    Operate<'a, Message, Theme, Renderer, State> for ()
 where
     Renderer: advanced::Renderer + 'a,
 {
     fn operate(
         &self,
         _state: &mut State,
-        inner: &Element<'a, Message, Theme, Renderer>,
+        inner: &mut Element<'a, Message, Theme, Renderer>,
         tree: &mut advanced::widget::Tree,
         layout: advanced::Layout<'_>,
         renderer: &Renderer,
         operation: &mut dyn advanced::widget::Operation<()>,
     ) {
-        inner.as_widget().operate(tree, layout, renderer, operation);
+        inner.as_widget_mut().operate(tree, layout, renderer, operation);
     }
 }
 
-impl<'a, T, Message, Theme, Renderer, State> Operate<'a, Message, Theme, Renderer, State> for T
+impl<'a, T, Message, Theme, Renderer, State>
+    Operate<'a, Message, Theme, Renderer, State> for T
 where
     T: Fn(
             &mut State,
-            &Element<'a, Message, Theme, Renderer>,
+            &mut Element<'a, Message, Theme, Renderer>,
             &mut advanced::widget::Tree,
             advanced::Layout<'_>,
             &Renderer,
@@ -524,7 +571,7 @@ where
     fn operate(
         &self,
         state: &mut State,
-        inner: &Element<'a, Message, Theme, Renderer>,
+        inner: &mut Element<'a, Message, Theme, Renderer>,
         tree: &mut advanced::widget::Tree,
         layout: advanced::Layout<'_>,
         renderer: &Renderer,
@@ -540,13 +587,15 @@ pub trait Overlay<'a, Message, Theme, Renderer, State> {
         state: &'b mut State,
         inner: &'b mut Element<'a, Message, Theme, Renderer>,
         tree: &'b mut advanced::widget::Tree,
-        layout: advanced::Layout<'_>,
+        layout: advanced::Layout<'b>,
         renderer: &Renderer,
+        viewport: &Rectangle,
         translation: iced::Vector,
     ) -> Option<advanced::overlay::Element<'b, Message, Theme, Renderer>>;
 }
 
-impl<'a, Message, Theme, Renderer, State> Overlay<'a, Message, Theme, Renderer, State> for ()
+impl<'a, Message, Theme, Renderer, State>
+    Overlay<'a, Message, Theme, Renderer, State> for ()
 where
     Renderer: advanced::Renderer + 'a,
 {
@@ -555,17 +604,23 @@ where
         _state: &'b mut State,
         inner: &'b mut Element<'a, Message, Theme, Renderer>,
         tree: &'b mut advanced::widget::Tree,
-        layout: advanced::Layout<'_>,
+        layout: advanced::Layout<'b>,
         renderer: &Renderer,
+        viewport: &Rectangle,
         translation: iced::Vector,
     ) -> Option<advanced::overlay::Element<'b, Message, Theme, Renderer>> {
-        inner
-            .as_widget_mut()
-            .overlay(tree, layout, renderer, translation)
+        inner.as_widget_mut().overlay(
+            tree,
+            layout,
+            renderer,
+            viewport,
+            translation,
+        )
     }
 }
 
-impl<'a, T, Message, Theme, Renderer, State> Overlay<'a, Message, Theme, Renderer, State> for T
+impl<'a, T, Message, Theme, Renderer, State>
+    Overlay<'a, Message, Theme, Renderer, State> for T
 where
     T: for<'b> Fn(
             &'b mut State,
@@ -574,8 +629,9 @@ where
             advanced::Layout<'_>,
             &Renderer,
             iced::Vector,
-        ) -> Option<advanced::overlay::Element<'b, Message, Theme, Renderer>>
-        + 'a,
+        ) -> Option<
+            advanced::overlay::Element<'b, Message, Theme, Renderer>,
+        > + 'a,
 {
     fn overlay<'b>(
         &'b mut self,
@@ -584,6 +640,7 @@ where
         tree: &'b mut advanced::widget::Tree,
         layout: advanced::Layout<'_>,
         renderer: &Renderer,
+        _viewport: &Rectangle,
         translation: iced::Vector,
     ) -> Option<advanced::overlay::Element<'b, Message, Theme, Renderer>> {
         self(state, inner, tree, layout, renderer, translation)
@@ -591,18 +648,18 @@ where
 }
 
 impl<
-        'a,
-        Message,
-        Theme,
-        Renderer,
-        Layout,
-        Update,
-        Draw,
-        MouseInteraction,
-        Operate,
-        Overlay,
-        State,
-    > Widget<Message, Theme, Renderer>
+    'a,
+    Message,
+    Theme,
+    Renderer,
+    Layout,
+    Update,
+    Draw,
+    MouseInteraction,
+    Operate,
+    Overlay,
+    State,
+> Widget<Message, Theme, Renderer>
     for Decorate<
         'a,
         Message,
@@ -621,7 +678,8 @@ where
     Layout: self::Layout<'a, Message, Theme, Renderer, State>,
     Update: self::Update<'a, Message, Theme, Renderer, State>,
     Draw: self::Draw<'a, Message, Theme, Renderer, State>,
-    MouseInteraction: self::MouseInteraction<'a, Message, Theme, Renderer, State> + 'a,
+    MouseInteraction:
+        self::MouseInteraction<'a, Message, Theme, Renderer, State> + 'a,
     Operate: self::Operate<'a, Message, Theme, Renderer, State> + 'a,
     Overlay: self::Overlay<'a, Message, Theme, Renderer, State> + 'a,
     State: Default + 'static,
@@ -652,14 +710,14 @@ where
     }
 
     fn layout(
-        &self,
+        &mut self,
         tree: &mut iced::advanced::widget::Tree,
         renderer: &Renderer,
         limits: &iced::advanced::layout::Limits,
     ) -> iced::advanced::layout::Node {
         self.layout.layout(
             tree.state.downcast_mut(),
-            &self.inner,
+            &mut self.inner,
             &mut tree.children[0],
             renderer,
             limits,
@@ -734,7 +792,7 @@ where
     }
 
     fn operate(
-        &self,
+        &mut self,
         tree: &mut advanced::widget::Tree,
         layout: advanced::Layout<'_>,
         renderer: &Renderer,
@@ -742,7 +800,7 @@ where
     ) {
         self.operate.operate(
             tree.state.downcast_mut(),
-            &self.inner,
+            &mut self.inner,
             &mut tree.children[0],
             layout,
             renderer,
@@ -753,8 +811,9 @@ where
     fn overlay<'b>(
         &'b mut self,
         tree: &'b mut advanced::widget::Tree,
-        layout: advanced::Layout<'_>,
+        layout: advanced::Layout<'b>,
         renderer: &Renderer,
+        viewport: &Rectangle,
         translation: iced::Vector,
     ) -> Option<advanced::overlay::Element<'b, Message, Theme, Renderer>> {
         self.overlay.overlay(
@@ -763,24 +822,25 @@ where
             &mut tree.children[0],
             layout,
             renderer,
+            viewport,
             translation,
         )
     }
 }
 
 impl<
-        'a,
-        Message,
-        Theme,
-        Renderer,
-        Layout,
-        Update,
-        Draw,
-        MouseInteraction,
-        Operate,
-        Overlay,
-        State,
-    >
+    'a,
+    Message,
+    Theme,
+    Renderer,
+    Layout,
+    Update,
+    Draw,
+    MouseInteraction,
+    Operate,
+    Overlay,
+    State,
+>
     From<
         Decorate<
             'a,
@@ -803,7 +863,8 @@ where
     Layout: self::Layout<'a, Message, Theme, Renderer, State> + 'a,
     Update: self::Update<'a, Message, Theme, Renderer, State> + 'a,
     Draw: self::Draw<'a, Message, Theme, Renderer, State> + 'a,
-    MouseInteraction: self::MouseInteraction<'a, Message, Theme, Renderer, State> + 'a,
+    MouseInteraction:
+        self::MouseInteraction<'a, Message, Theme, Renderer, State> + 'a,
     Operate: self::Operate<'a, Message, Theme, Renderer, State> + 'a,
     Overlay: self::Overlay<'a, Message, Theme, Renderer, State> + 'a,
     State: Default + 'static,
