@@ -1,13 +1,12 @@
 use iced::advanced::layout::{self, Layout};
-use iced::advanced::overlay;
-use iced::advanced::renderer;
 use iced::advanced::widget::{self, Widget};
-use iced::advanced::{self, Clipboard, Shell};
+use iced::advanced::{self, Clipboard, Shell, overlay, renderer};
 use iced::alignment::Alignment;
-use iced::keyboard;
 use iced::keyboard::key;
-use iced::mouse;
-use iced::{Color, Element, Event, Length, Point, Rectangle, Size, Vector};
+use iced::{
+    Color, Element, Event, Length, Point, Rectangle, Size, Vector, keyboard,
+    mouse,
+};
 
 pub fn modal<'a, Message, Theme, Renderer>(
     base: impl Into<Element<'a, Message, Theme, Renderer>>,
@@ -65,13 +64,13 @@ where
     }
 
     fn layout(
-        &self,
+        &mut self,
         tree: &mut widget::Tree,
         renderer: &Renderer,
         limits: &layout::Limits,
     ) -> layout::Node {
         self.base
-            .as_widget()
+            .as_widget_mut()
             .layout(&mut tree.children[0], renderer, limits)
     }
 
@@ -122,8 +121,9 @@ where
     fn overlay<'b>(
         &'b mut self,
         state: &'b mut widget::Tree,
-        layout: Layout<'_>,
+        layout: Layout<'b>,
         _renderer: &Renderer,
+        viewport: &Rectangle,
         translation: Vector,
     ) -> Option<overlay::Element<'b, Message, Theme, Renderer>> {
         Some(overlay::Element::new(Box::new(Overlay {
@@ -132,6 +132,7 @@ where
             tree: &mut state.children[1],
             size: layout.bounds().size(),
             on_blur: &self.on_blur,
+            viewport: *viewport,
         })))
     }
 
@@ -153,15 +154,18 @@ where
     }
 
     fn operate(
-        &self,
+        &mut self,
         state: &mut widget::Tree,
         layout: Layout<'_>,
         renderer: &Renderer,
         operation: &mut dyn widget::Operation<()>,
     ) {
-        self.base
-            .as_widget()
-            .operate(&mut state.children[0], layout, renderer, operation);
+        self.base.as_widget_mut().operate(
+            &mut state.children[0],
+            layout,
+            renderer,
+            operation,
+        );
     }
 }
 
@@ -171,6 +175,7 @@ struct Overlay<'a, 'b, Message, Theme, Renderer> {
     tree: &'b mut widget::Tree,
     size: Size,
     on_blur: &'b dyn Fn() -> Message,
+    viewport: Rectangle,
 }
 
 impl<Message, Theme, Renderer> overlay::Overlay<Message, Theme, Renderer>
@@ -185,11 +190,12 @@ where
 
         let child = self
             .content
-            .as_widget()
+            .as_widget_mut()
             .layout(self.tree, renderer, &limits)
             .align(Alignment::Center, Alignment::Center, limits.max());
 
-        layout::Node::with_children(self.size, vec![child]).move_to(self.position)
+        layout::Node::with_children(self.size, vec![child])
+            .move_to(self.position)
     }
 
     fn update(
@@ -270,7 +276,7 @@ where
         renderer: &Renderer,
         operation: &mut dyn widget::Operation<()>,
     ) {
-        self.content.as_widget().operate(
+        self.content.as_widget_mut().operate(
             self.tree,
             layout.children().next().unwrap(),
             renderer,
@@ -282,27 +288,27 @@ where
         &self,
         layout: Layout<'_>,
         cursor: mouse::Cursor,
-        viewport: &Rectangle,
         renderer: &Renderer,
     ) -> mouse::Interaction {
         self.content.as_widget().mouse_interaction(
             self.tree,
             layout.children().next().unwrap(),
             cursor,
-            viewport,
+            &layout.bounds(),
             renderer,
         )
     }
 
     fn overlay<'c>(
         &'c mut self,
-        layout: Layout<'_>,
+        layout: Layout<'c>,
         renderer: &Renderer,
     ) -> Option<overlay::Element<'c, Message, Theme, Renderer>> {
         self.content.as_widget_mut().overlay(
             self.tree,
             layout.children().next().unwrap(),
             renderer,
+            &self.viewport,
             Vector::ZERO,
         )
     }
