@@ -2,10 +2,11 @@ use iced::advanced::renderer::Quad;
 use iced::advanced::text::{Paragraph, paragraph};
 use iced::advanced::widget::{Operation, Tree, operation, tree};
 use iced::advanced::{Layout, Widget, layout, mouse, renderer, text, widget};
-use iced::widget::text::{Fragment, IntoFragment, Wrapping};
+use iced::widget::text::{Format, Fragment, IntoFragment, Wrapping};
 use iced::widget::text_input::Value;
 use iced::{
-    Border, Color, Element, Length, Pixels, Point, Rectangle, Shadow, Size, Task, alignment, touch,
+    Border, Color, Element, Length, Pixels, Point, Rectangle, Shadow, Size,
+    Task, alignment, touch,
 };
 
 pub use self::selection::selection;
@@ -29,15 +30,7 @@ where
     Theme: Catalog,
 {
     fragment: Fragment<'a>,
-    size: Option<Pixels>,
-    line_height: LineHeight,
-    width: Length,
-    height: Length,
-    align_x: text::Alignment,
-    align_y: alignment::Vertical,
-    font: Option<Renderer::Font>,
-    shaping: Shaping,
-    wrapping: Wrapping,
+    format: Format<Renderer::Font>,
     class: Theme::Class<'a>,
 }
 
@@ -49,34 +42,38 @@ where
     pub fn new(fragment: impl IntoFragment<'a>) -> Self {
         Text {
             fragment: fragment.into_fragment(),
-            size: None,
-            line_height: LineHeight::default(),
-            font: None,
-            width: Length::Shrink,
-            height: Length::Shrink,
-            align_x: text::Alignment::Left,
-            align_y: alignment::Vertical::Top,
-            #[cfg(debug_assertions)]
-            shaping: Shaping::Basic,
-            #[cfg(not(debug_assertions))]
-            shaping: Shaping::Advanced,
-            wrapping: Wrapping::default(),
+            format: Format {
+                #[cfg(debug_assertions)]
+                shaping: Shaping::Basic,
+                #[cfg(not(debug_assertions))]
+                shaping: Shaping::Advanced,
+                wrapping: Wrapping::WordOrGlyph,
+                ..Format::default()
+            },
             class: Theme::default(),
         }
     }
 
     pub fn size(mut self, size: impl Into<Pixels>) -> Self {
-        self.size = Some(size.into());
+        self.format.size = Some(size.into());
         self
     }
 
     pub fn line_height(mut self, line_height: impl Into<LineHeight>) -> Self {
-        self.line_height = line_height.into();
+        self.format.line_height = line_height.into();
         self
     }
 
     pub fn font(mut self, font: impl Into<Renderer::Font>) -> Self {
-        self.font = Some(font.into());
+        self.format.font = Some(font.into());
+        self
+    }
+
+    pub fn font_maybe(
+        mut self,
+        font: Option<impl Into<Renderer::Font>>,
+    ) -> Self {
+        self.format.font = font.map(Into::into);
         self
     }
 
@@ -89,27 +86,27 @@ where
     }
 
     pub fn width(mut self, width: impl Into<Length>) -> Self {
-        self.width = width.into();
+        self.format.width = width.into();
         self
     }
 
     pub fn height(mut self, height: impl Into<Length>) -> Self {
-        self.height = height.into();
+        self.format.height = height.into();
         self
     }
 
     pub fn align_x(mut self, alignment: text::Alignment) -> Self {
-        self.align_x = alignment;
+        self.format.align_x = alignment;
         self
     }
 
     pub fn align_y(mut self, alignment: alignment::Vertical) -> Self {
-        self.align_y = alignment;
+        self.format.align_y = alignment;
         self
     }
 
     pub fn shaping(mut self, shaping: Shaping) -> Self {
-        self.shaping = shaping;
+        self.format.shaping = shaping;
         self
     }
 
@@ -119,15 +116,16 @@ where
     }
 }
 
-impl<Message, Theme, Renderer> Widget<Message, Theme, Renderer> for Text<'_, Theme, Renderer>
+impl<Message, Theme, Renderer> Widget<Message, Theme, Renderer>
+    for Text<'_, Theme, Renderer>
 where
     Renderer: text::Renderer,
     Theme: Catalog,
 {
     fn size(&self) -> Size<Length> {
         Size {
-            width: self.width,
-            height: self.height,
+            width: self.format.width,
+            height: self.format.height,
         }
     }
 
@@ -140,29 +138,31 @@ where
     }
 
     fn layout(
-        &self,
+        &mut self,
         tree: &mut Tree,
         renderer: &Renderer,
         limits: &layout::Limits,
     ) -> layout::Node {
         let state = tree.state.downcast_mut::<State<Renderer::Paragraph>>();
 
-        layout::sized(limits, self.width, self.height, |limits| {
+        layout::sized(limits, self.format.width, self.format.height, |limits| {
             let bounds = limits.max();
 
-            let size = self.size.unwrap_or_else(|| renderer.default_size());
-            let font = self.font.unwrap_or_else(|| renderer.default_font());
+            let size =
+                self.format.size.unwrap_or_else(|| renderer.default_size());
+            let font =
+                self.format.font.unwrap_or_else(|| renderer.default_font());
 
             state.paragraph.update(text::Text {
                 content: &self.fragment,
-                size,
-                line_height: self.line_height,
                 bounds,
+                size,
+                line_height: self.format.line_height,
                 font,
-                align_x: self.align_x,
-                align_y: self.align_y,
-                shaping: self.shaping,
-                wrapping: self.wrapping,
+                align_x: self.format.align_x,
+                align_y: self.format.align_y,
+                shaping: self.format.shaping,
+                wrapping: self.format.wrapping,
             });
 
             state.paragraph.min_bounds()
@@ -189,18 +189,23 @@ where
         state.hovered = cursor.is_over(bounds);
 
         match event {
-            iced::Event::Mouse(mouse::Event::ButtonPressed(mouse::Button::Left))
+            iced::Event::Mouse(mouse::Event::ButtonPressed(
+                mouse::Button::Left,
+            ))
             | iced::Event::Touch(touch::Event::FingerPressed { .. }) => {
                 if let Some(cursor) = cursor.position() {
-                    state.interaction = Interaction::Selecting(selection::Raw {
-                        start: cursor,
-                        end: cursor,
-                    });
+                    state.interaction =
+                        Interaction::Selecting(selection::Raw {
+                            start: cursor,
+                            end: cursor,
+                        });
                 } else {
                     state.interaction = Interaction::Idle;
                 }
             }
-            iced::Event::Mouse(mouse::Event::ButtonReleased(mouse::Button::Left))
+            iced::Event::Mouse(mouse::Event::ButtonReleased(
+                mouse::Button::Left,
+            ))
             | iced::Event::Touch(touch::Event::FingerLifted { .. })
             | iced::Event::Touch(touch::Event::FingerLost { .. }) => {
                 if let Interaction::Selecting(raw) = state.interaction {
@@ -211,16 +216,18 @@ where
             }
             iced::Event::Mouse(mouse::Event::CursorMoved { .. })
             | iced::Event::Touch(touch::Event::FingerMoved { .. }) => {
-                if let Some(cursor) = cursor.position() {
-                    if let Interaction::Selecting(raw) = &mut state.interaction {
+                if let Some(cursor) = cursor.position()
+                    && let Interaction::Selecting(raw) = &mut state.interaction
+                    {
                         raw.end = cursor;
                     }
-                }
             }
             _ => {}
         }
 
-        if prev_hovered != state.hovered || prev_interaction != state.interaction {
+        if prev_hovered != state.hovered
+            || prev_interaction != state.interaction
+        {
             shell.request_redraw();
         }
     }
@@ -250,13 +257,13 @@ where
             .selection()
             .and_then(|raw| raw.resolve(bounds))
         {
-            let line_height = f32::from(
-                self.line_height
-                    .to_absolute(self.size.unwrap_or_else(|| renderer.default_size())),
-            );
+            let line_height = f32::from(self.format.line_height.to_absolute(
+                self.format.size.unwrap_or_else(|| renderer.default_size()),
+            ));
 
-            let baseline_y =
-                bounds.y + ((selection.start.y - bounds.y) / line_height).floor() * line_height;
+            let baseline_y = bounds.y
+                + ((selection.start.y - bounds.y) / line_height).floor()
+                    * line_height;
 
             let height = selection.end.y - baseline_y - 0.5;
             let rows = (height / line_height).ceil() as usize;
@@ -266,7 +273,8 @@ where
                     (
                         selection.start.x,
                         if rows == 1 {
-                            f32::min(selection.end.x, bounds.x + bounds.width) - selection.start.x
+                            f32::min(selection.end.x, bounds.x + bounds.width)
+                                - selection.start.x
                         } else {
                             bounds.x + bounds.width - selection.start.x
                         },
@@ -280,13 +288,17 @@ where
 
                 renderer.fill_quad(
                     Quad {
-                        bounds: Rectangle::new(Point::new(x, y), Size::new(width, line_height)),
+                        bounds: Rectangle::new(
+                            Point::new(x, y),
+                            Size::new(width, line_height),
+                        ),
                         border: Border {
                             radius: 0.0.into(),
                             width: 0.0,
                             color: Color::TRANSPARENT,
                         },
                         shadow: Shadow::default(),
+                        snap: true,
                     },
                     appearance.selection_color,
                 );
@@ -382,7 +394,7 @@ where
     }
 
     fn operate(
-        &self,
+        &mut self,
         tree: &mut Tree,
         layout: Layout<'_>,
         _renderer: &Renderer,
@@ -392,12 +404,11 @@ where
 
         let bounds = layout.bounds();
         let value = Value::new(&self.fragment);
-        if let Some(selection) = state
-            .interaction
-            .selection()
-            .and_then(|raw| selection(raw, bounds, state.paragraph.raw(), &value))
-        {
-            let mut content = value.select(selection.start, selection.end).to_string();
+        if let Some(selection) = state.interaction.selection().and_then(|raw| {
+            selection(raw, bounds, state.paragraph.raw(), &value)
+        }) {
+            let mut content =
+                value.select(selection.start, selection.end).to_string();
             operation.custom(None, bounds, &mut content);
         }
     }
@@ -414,23 +425,15 @@ fn draw<Renderer>(
     Renderer: text::Renderer,
 {
     let State { paragraph, .. } = &state;
-    let bounds = layout.bounds();
-
-    let x = match paragraph.align_x() {
-        text::Alignment::Default | text::Alignment::Left | text::Alignment::Justified => bounds.x,
-        text::Alignment::Center => bounds.center_x(),
-        text::Alignment::Right => bounds.x + bounds.width,
-    };
-
-    let y = match paragraph.align_y() {
-        alignment::Vertical::Top => bounds.y,
-        alignment::Vertical::Center => bounds.center_y(),
-        alignment::Vertical::Bottom => bounds.y + bounds.height,
-    };
+    let anchor = layout.bounds().anchor(
+        paragraph.min_bounds(),
+        paragraph.align_x(),
+        paragraph.align_y(),
+    );
 
     renderer.fill_paragraph(
         paragraph.raw(),
-        Point::new(x, y),
+        anchor,
         appearance.color.unwrap_or(style.text_color),
         *viewport,
     );
@@ -442,7 +445,9 @@ where
     Renderer: text::Renderer + 'a,
     Theme: Catalog + 'a,
 {
-    fn from(text: Text<'a, Theme, Renderer>) -> Element<'a, Message, Theme, Renderer> {
+    fn from(
+        text: Text<'a, Theme, Renderer>,
+    ) -> Element<'a, Message, Theme, Renderer> {
         Element::new(text)
     }
 }
@@ -466,7 +471,9 @@ impl Interaction {
     pub fn selection(self) -> Option<selection::Raw> {
         match &self {
             Interaction::Idle => None,
-            Interaction::Selecting(raw) | Interaction::Selected(raw) => Some(*raw),
+            Interaction::Selecting(raw) | Interaction::Selected(raw) => {
+                Some(*raw)
+            }
         }
     }
 }
@@ -486,7 +493,9 @@ impl Interaction {
 //     renderer.measure_width(&value.to_string(), size, font, text::Shaping::Advanced)
 // }
 
-pub fn selected<Message: Send + 'static>(f: fn(Vec<(f32, String)>) -> Message) -> Task<Message> {
+pub fn selected<Message: Send + 'static>(
+    f: fn(Vec<(f32, String)>) -> Message,
+) -> Task<Message> {
     struct Selected<T> {
         contents: Vec<(f32, String)>,
         f: fn(Vec<(f32, String)>) -> T,
